/***************************************** <lalVerbatim file="GridCV">
Author: Creighton, T. D.
$Id$
**************************************************** </lalVerbatim> */

/********************************************************** <lalLaTeX>

\subsection{Module \texttt{Grid.c}}
\label{ss:Grid.c}

Creates or destroys a LAL grid structure.

\subsubsection*{Prototypes}
\vspace{0.1in}
\begin{verbatim}
void
LAL<typecode>CreateGrid( LALStatus      *stat,
                         <datatype>Grid **grid,
                         UINT4Vector    *dimLength,
                         UINT4          dimension )

void
LAL<typecode>DestroyGrid( LALStatus      *stat,
                          <datatype>Grid **grid )
\end{verbatim}

\idx{LALI2CreateGrid()}
\idx{LALI4CreateGrid()}
\idx{LALI8CreateGrid()}
\idx{LALU2CreateGrid()}
\idx{LALU4CreateGrid()}
\idx{LALU8CreateGrid()}
\idx{LALSCreateGrid()}
\idx{LALDCreateGrid()}
\idx{LALCCreateGrid()}
\idx{LALZCreateGrid()}
\idx{LALI2DestroyGrid()}
\idx{LALI4DestroyGrid()}
\idx{LALI8DestroyGrid()}
\idx{LALU2DestroyGrid()}
\idx{LALU4DestroyGrid()}
\idx{LALU8DestroyGrid()}
\idx{LALSDestroyGrid()}
\idx{LALDDestroyGrid()}
\idx{LALCDestroyGrid()}
\idx{LALZDestroyGrid()}

\subsubsection*{Description}

These routines create or destroy a \verb@<datatype>Grid@ structure.
The input vector \verb@dimLength@ stores the lengths of each dimension
of the grid \emph{and} of the array at each grid point: in the
notation defined in \verb@Grid.h@, \verb@dimLength->length@$=M$.  The
parameter \verb@dimension@ gives the dimension $m$ of the physical
grid space; if $M>m$, then the remaining dimensions refer to a tangent
space at each grid point.  When creating a grid, the routines allocate
space for all the internal vectors and arrays, but no data are filled
in, with the exception of the \verb@(*grid)->data->dimLength@ vector
(which will contain exactly the same data as the \verb@dimLength@
input parameter).  When calling the \verb@LAL<typecode>CreateGrid()@
routines, or on returning from the \verb@LAL<typecode>DestroyGrid()@
routines, \verb@grid@ should be a non-\verb@NULL@ handle to a
\verb@NULL@-valued pointer.

For each of these prototype templates there are in fact 10 separate
routines corresponding to all the numerical atomic datatypes
\verb@<datatype>@ referred to by \verb@<typecode>@:
\begin{center}
\begin{tabular}{|c@{\qquad}c|c@{\qquad}c|}
\hline
\tt <typecode> & \tt <datatype> & \tt <typecode> & \tt <datatype> \\
\hline
\tt I2 & \tt  INT2 & \tt U2 & \tt    UINT2  \\
\tt I4 & \tt  INT4 & \tt U4 & \tt    UINT4  \\
\tt I8 & \tt  INT8 & \tt U8 & \tt    UINT8  \\
\tt  S & \tt REAL4 & \tt  C & \tt COMPLEX8  \\
\tt  D & \tt REAL8 & \tt  Z & \tt COMPLEX16 \\
\hline
\end{tabular}
\end{center}

\subsubsection*{Algorithm}

\subsubsection*{Uses}
\begin{verbatim}
lalDebugLevel
LALMalloc()                     LALFree()
LALDCreateVector()              LALDDestroyVector()
LAL<typecode>CreateArray()      LAL<typecode>DestroyArray()
\end{verbatim}

\subsubsection*{Notes}

\vfill{\footnotesize\input{GridCV}}

******************************************************* </lalLaTeX> */

#include <lal/LALStdlib.h>
#include <lal/AVFactories.h>
#include <lal/Grid.h>

NRCSID( GRIDC, "$Id$" );


void
LALI2CreateGrid ( LALStatus *stat, INT2Grid **grid, UINT4Vector *dimLength, UINT4 dimension )
{
  INITSTATUS( stat, "LALI2CreateGrid", GRIDC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( dimLength, stat, GRIDH_ENUL, GRIDH_MSGENUL );
  ASSERT( dimLength->data, stat, GRIDH_ENUL, GRIDH_MSGENUL );
  ASSERT( grid, stat, GRIDH_ENUL, GRIDH_MSGENUL );
  ASSERT( !*grid, stat, GRIDH_EOUT, GRIDH_MSGEOUT );
  ASSERT( dimension <= dimLength->length, stat, GRIDH_ENUL,
	  GRIDH_MSGENUL );

  /* Allocate the grid. */
  if ( !( *grid = ( INT2Grid *)LALMalloc( sizeof( INT2Grid ) ) ) ) {
    ABORT( stat, GRIDH_EMEM, GRIDH_MSGEMEM );
  }
  memset( *grid, 0, sizeof( INT2Grid ) );

  /* Allocate the dimension units array. */
  if ( !( (*grid)->dimUnits = (LALUnit *)
	  LALMalloc( dimension*sizeof(LALUnit) ) ) ) {
    LALFree( *grid );
    *grid = NULL;
    ABORT( stat, GRIDH_EMEM, GRIDH_MSGEMEM );
  }
  memset( (*grid)->dimUnits, 0, dimension*sizeof(LALUnit) );

  /* Allocate the offset and interval vectors. */
  LALDCreateVector( stat->statusPtr, &((*grid)->offset), dimension );
  BEGINFAIL( stat ) {
    LALFree( (*grid)->dimUnits );
    LALFree( *grid );
    *grid = NULL;
  } ENDFAIL( stat );
  LALDCreateVector( stat->statusPtr, &((*grid)->interval), dimension );
  BEGINFAIL( stat ) {
    TRY( LALDDestroyVector( stat->statusPtr, &((*grid)->offset) ), stat );
    LALFree( (*grid)->dimUnits );
    LALFree( *grid );
    *grid = NULL;
  } ENDFAIL( stat );

  /* Allocate the data array. */
  LALI2CreateArray ( stat->statusPtr, &((*grid)->data), dimLength );
  BEGINFAIL( stat ) {
    TRY( LALDDestroyVector( stat->statusPtr, &((*grid)->interval) ), stat );
    TRY( LALDDestroyVector( stat->statusPtr, &((*grid)->offset) ), stat );
    LALFree( (*grid)->dimUnits );
    LALFree( *grid );
    *grid = NULL;
  } ENDFAIL( stat );

  /* Done. */
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}


void
LALI2DestroyGrid ( LALStatus *stat, INT2Grid **grid )
{
  INITSTATUS( stat, "LALI2DestroyGrid", GRIDC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input argument. */
  ASSERT( grid, stat, GRIDH_ENUL, GRIDH_MSGENUL );
  ASSERT( *grid, stat, GRIDH_ENUL, GRIDH_MSGENUL );

  /* Destroy the internal arrays and vectors. */
  TRY( LALI2DestroyArray ( stat->statusPtr, &((*grid)->data) ), stat );
  TRY( LALDDestroyVector( stat->statusPtr, &((*grid)->interval) ), stat );
  TRY( LALDDestroyVector( stat->statusPtr, &((*grid)->offset) ), stat );
  LALFree( (*grid)->dimUnits );

  /* Destroy the structure itself, and exit. */
  LALFree( *grid );
  *grid = NULL;
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}


void
LALI4CreateGrid ( LALStatus *stat, INT4Grid **grid, UINT4Vector *dimLength, UINT4 dimension )
{
  INITSTATUS( stat, "LALI4CreateGrid", GRIDC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( dimLength, stat, GRIDH_ENUL, GRIDH_MSGENUL );
  ASSERT( dimLength->data, stat, GRIDH_ENUL, GRIDH_MSGENUL );
  ASSERT( grid, stat, GRIDH_ENUL, GRIDH_MSGENUL );
  ASSERT( !*grid, stat, GRIDH_EOUT, GRIDH_MSGEOUT );
  ASSERT( dimension <= dimLength->length, stat, GRIDH_ENUL,
	  GRIDH_MSGENUL );

  /* Allocate the grid. */
  if ( !( *grid = ( INT4Grid *)LALMalloc( sizeof( INT4Grid ) ) ) ) {
    ABORT( stat, GRIDH_EMEM, GRIDH_MSGEMEM );
  }
  memset( *grid, 0, sizeof( INT4Grid ) );

  /* Allocate the dimension units array. */
  if ( !( (*grid)->dimUnits = (LALUnit *)
	  LALMalloc( dimension*sizeof(LALUnit) ) ) ) {
    LALFree( *grid );
    *grid = NULL;
    ABORT( stat, GRIDH_EMEM, GRIDH_MSGEMEM );
  }
  memset( (*grid)->dimUnits, 0, dimension*sizeof(LALUnit) );

  /* Allocate the offset and interval vectors. */
  LALDCreateVector( stat->statusPtr, &((*grid)->offset), dimension );
  BEGINFAIL( stat ) {
    LALFree( (*grid)->dimUnits );
    LALFree( *grid );
    *grid = NULL;
  } ENDFAIL( stat );
  LALDCreateVector( stat->statusPtr, &((*grid)->interval), dimension );
  BEGINFAIL( stat ) {
    TRY( LALDDestroyVector( stat->statusPtr, &((*grid)->offset) ), stat );
    LALFree( (*grid)->dimUnits );
    LALFree( *grid );
    *grid = NULL;
  } ENDFAIL( stat );

  /* Allocate the data array. */
  LALI4CreateArray ( stat->statusPtr, &((*grid)->data), dimLength );
  BEGINFAIL( stat ) {
    TRY( LALDDestroyVector( stat->statusPtr, &((*grid)->interval) ), stat );
    TRY( LALDDestroyVector( stat->statusPtr, &((*grid)->offset) ), stat );
    LALFree( (*grid)->dimUnits );
    LALFree( *grid );
    *grid = NULL;
  } ENDFAIL( stat );

  /* Done. */
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}


void
LALI4DestroyGrid ( LALStatus *stat, INT4Grid **grid )
{
  INITSTATUS( stat, "LALI4DestroyGrid", GRIDC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input argument. */
  ASSERT( grid, stat, GRIDH_ENUL, GRIDH_MSGENUL );
  ASSERT( *grid, stat, GRIDH_ENUL, GRIDH_MSGENUL );

  /* Destroy the internal arrays and vectors. */
  TRY( LALI4DestroyArray ( stat->statusPtr, &((*grid)->data) ), stat );
  TRY( LALDDestroyVector( stat->statusPtr, &((*grid)->interval) ), stat );
  TRY( LALDDestroyVector( stat->statusPtr, &((*grid)->offset) ), stat );
  LALFree( (*grid)->dimUnits );

  /* Destroy the structure itself, and exit. */
  LALFree( *grid );
  *grid = NULL;
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}


void
LALI8CreateGrid ( LALStatus *stat, INT8Grid **grid, UINT4Vector *dimLength, UINT4 dimension )
{
  INITSTATUS( stat, "LALI8CreateGrid", GRIDC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( dimLength, stat, GRIDH_ENUL, GRIDH_MSGENUL );
  ASSERT( dimLength->data, stat, GRIDH_ENUL, GRIDH_MSGENUL );
  ASSERT( grid, stat, GRIDH_ENUL, GRIDH_MSGENUL );
  ASSERT( !*grid, stat, GRIDH_EOUT, GRIDH_MSGEOUT );
  ASSERT( dimension <= dimLength->length, stat, GRIDH_ENUL,
	  GRIDH_MSGENUL );

  /* Allocate the grid. */
  if ( !( *grid = ( INT8Grid *)LALMalloc( sizeof( INT8Grid ) ) ) ) {
    ABORT( stat, GRIDH_EMEM, GRIDH_MSGEMEM );
  }
  memset( *grid, 0, sizeof( INT8Grid ) );

  /* Allocate the dimension units array. */
  if ( !( (*grid)->dimUnits = (LALUnit *)
	  LALMalloc( dimension*sizeof(LALUnit) ) ) ) {
    LALFree( *grid );
    *grid = NULL;
    ABORT( stat, GRIDH_EMEM, GRIDH_MSGEMEM );
  }
  memset( (*grid)->dimUnits, 0, dimension*sizeof(LALUnit) );

  /* Allocate the offset and interval vectors. */
  LALDCreateVector( stat->statusPtr, &((*grid)->offset), dimension );
  BEGINFAIL( stat ) {
    LALFree( (*grid)->dimUnits );
    LALFree( *grid );
    *grid = NULL;
  } ENDFAIL( stat );
  LALDCreateVector( stat->statusPtr, &((*grid)->interval), dimension );
  BEGINFAIL( stat ) {
    TRY( LALDDestroyVector( stat->statusPtr, &((*grid)->offset) ), stat );
    LALFree( (*grid)->dimUnits );
    LALFree( *grid );
    *grid = NULL;
  } ENDFAIL( stat );

  /* Allocate the data array. */
  LALI8CreateArray ( stat->statusPtr, &((*grid)->data), dimLength );
  BEGINFAIL( stat ) {
    TRY( LALDDestroyVector( stat->statusPtr, &((*grid)->interval) ), stat );
    TRY( LALDDestroyVector( stat->statusPtr, &((*grid)->offset) ), stat );
    LALFree( (*grid)->dimUnits );
    LALFree( *grid );
    *grid = NULL;
  } ENDFAIL( stat );

  /* Done. */
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}


void
LALI8DestroyGrid ( LALStatus *stat, INT8Grid **grid )
{
  INITSTATUS( stat, "LALI8DestroyGrid", GRIDC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input argument. */
  ASSERT( grid, stat, GRIDH_ENUL, GRIDH_MSGENUL );
  ASSERT( *grid, stat, GRIDH_ENUL, GRIDH_MSGENUL );

  /* Destroy the internal arrays and vectors. */
  TRY( LALI8DestroyArray ( stat->statusPtr, &((*grid)->data) ), stat );
  TRY( LALDDestroyVector( stat->statusPtr, &((*grid)->interval) ), stat );
  TRY( LALDDestroyVector( stat->statusPtr, &((*grid)->offset) ), stat );
  LALFree( (*grid)->dimUnits );

  /* Destroy the structure itself, and exit. */
  LALFree( *grid );
  *grid = NULL;
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}


void
LALU2CreateGrid ( LALStatus *stat, UINT2Grid **grid, UINT4Vector *dimLength, UINT4 dimension )
{
  INITSTATUS( stat, "LALU2CreateGrid", GRIDC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( dimLength, stat, GRIDH_ENUL, GRIDH_MSGENUL );
  ASSERT( dimLength->data, stat, GRIDH_ENUL, GRIDH_MSGENUL );
  ASSERT( grid, stat, GRIDH_ENUL, GRIDH_MSGENUL );
  ASSERT( !*grid, stat, GRIDH_EOUT, GRIDH_MSGEOUT );
  ASSERT( dimension <= dimLength->length, stat, GRIDH_ENUL,
	  GRIDH_MSGENUL );

  /* Allocate the grid. */
  if ( !( *grid = ( UINT2Grid *)LALMalloc( sizeof( UINT2Grid ) ) ) ) {
    ABORT( stat, GRIDH_EMEM, GRIDH_MSGEMEM );
  }
  memset( *grid, 0, sizeof( UINT2Grid ) );

  /* Allocate the dimension units array. */
  if ( !( (*grid)->dimUnits = (LALUnit *)
	  LALMalloc( dimension*sizeof(LALUnit) ) ) ) {
    LALFree( *grid );
    *grid = NULL;
    ABORT( stat, GRIDH_EMEM, GRIDH_MSGEMEM );
  }
  memset( (*grid)->dimUnits, 0, dimension*sizeof(LALUnit) );

  /* Allocate the offset and interval vectors. */
  LALDCreateVector( stat->statusPtr, &((*grid)->offset), dimension );
  BEGINFAIL( stat ) {
    LALFree( (*grid)->dimUnits );
    LALFree( *grid );
    *grid = NULL;
  } ENDFAIL( stat );
  LALDCreateVector( stat->statusPtr, &((*grid)->interval), dimension );
  BEGINFAIL( stat ) {
    TRY( LALDDestroyVector( stat->statusPtr, &((*grid)->offset) ), stat );
    LALFree( (*grid)->dimUnits );
    LALFree( *grid );
    *grid = NULL;
  } ENDFAIL( stat );

  /* Allocate the data array. */
  LALU2CreateArray ( stat->statusPtr, &((*grid)->data), dimLength );
  BEGINFAIL( stat ) {
    TRY( LALDDestroyVector( stat->statusPtr, &((*grid)->interval) ), stat );
    TRY( LALDDestroyVector( stat->statusPtr, &((*grid)->offset) ), stat );
    LALFree( (*grid)->dimUnits );
    LALFree( *grid );
    *grid = NULL;
  } ENDFAIL( stat );

  /* Done. */
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}


void
LALU2DestroyGrid ( LALStatus *stat, UINT2Grid **grid )
{
  INITSTATUS( stat, "LALU2DestroyGrid", GRIDC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input argument. */
  ASSERT( grid, stat, GRIDH_ENUL, GRIDH_MSGENUL );
  ASSERT( *grid, stat, GRIDH_ENUL, GRIDH_MSGENUL );

  /* Destroy the internal arrays and vectors. */
  TRY( LALU2DestroyArray ( stat->statusPtr, &((*grid)->data) ), stat );
  TRY( LALDDestroyVector( stat->statusPtr, &((*grid)->interval) ), stat );
  TRY( LALDDestroyVector( stat->statusPtr, &((*grid)->offset) ), stat );
  LALFree( (*grid)->dimUnits );

  /* Destroy the structure itself, and exit. */
  LALFree( *grid );
  *grid = NULL;
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}


void
LALU4CreateGrid ( LALStatus *stat, UINT4Grid **grid, UINT4Vector *dimLength, UINT4 dimension )
{
  INITSTATUS( stat, "LALU4CreateGrid", GRIDC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( dimLength, stat, GRIDH_ENUL, GRIDH_MSGENUL );
  ASSERT( dimLength->data, stat, GRIDH_ENUL, GRIDH_MSGENUL );
  ASSERT( grid, stat, GRIDH_ENUL, GRIDH_MSGENUL );
  ASSERT( !*grid, stat, GRIDH_EOUT, GRIDH_MSGEOUT );
  ASSERT( dimension <= dimLength->length, stat, GRIDH_ENUL,
	  GRIDH_MSGENUL );

  /* Allocate the grid. */
  if ( !( *grid = ( UINT4Grid *)LALMalloc( sizeof( UINT4Grid ) ) ) ) {
    ABORT( stat, GRIDH_EMEM, GRIDH_MSGEMEM );
  }
  memset( *grid, 0, sizeof( UINT4Grid ) );

  /* Allocate the dimension units array. */
  if ( !( (*grid)->dimUnits = (LALUnit *)
	  LALMalloc( dimension*sizeof(LALUnit) ) ) ) {
    LALFree( *grid );
    *grid = NULL;
    ABORT( stat, GRIDH_EMEM, GRIDH_MSGEMEM );
  }
  memset( (*grid)->dimUnits, 0, dimension*sizeof(LALUnit) );

  /* Allocate the offset and interval vectors. */
  LALDCreateVector( stat->statusPtr, &((*grid)->offset), dimension );
  BEGINFAIL( stat ) {
    LALFree( (*grid)->dimUnits );
    LALFree( *grid );
    *grid = NULL;
  } ENDFAIL( stat );
  LALDCreateVector( stat->statusPtr, &((*grid)->interval), dimension );
  BEGINFAIL( stat ) {
    TRY( LALDDestroyVector( stat->statusPtr, &((*grid)->offset) ), stat );
    LALFree( (*grid)->dimUnits );
    LALFree( *grid );
    *grid = NULL;
  } ENDFAIL( stat );

  /* Allocate the data array. */
  LALU4CreateArray ( stat->statusPtr, &((*grid)->data), dimLength );
  BEGINFAIL( stat ) {
    TRY( LALDDestroyVector( stat->statusPtr, &((*grid)->interval) ), stat );
    TRY( LALDDestroyVector( stat->statusPtr, &((*grid)->offset) ), stat );
    LALFree( (*grid)->dimUnits );
    LALFree( *grid );
    *grid = NULL;
  } ENDFAIL( stat );

  /* Done. */
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}


void
LALU4DestroyGrid ( LALStatus *stat, UINT4Grid **grid )
{
  INITSTATUS( stat, "LALU4DestroyGrid", GRIDC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input argument. */
  ASSERT( grid, stat, GRIDH_ENUL, GRIDH_MSGENUL );
  ASSERT( *grid, stat, GRIDH_ENUL, GRIDH_MSGENUL );

  /* Destroy the internal arrays and vectors. */
  TRY( LALU4DestroyArray ( stat->statusPtr, &((*grid)->data) ), stat );
  TRY( LALDDestroyVector( stat->statusPtr, &((*grid)->interval) ), stat );
  TRY( LALDDestroyVector( stat->statusPtr, &((*grid)->offset) ), stat );
  LALFree( (*grid)->dimUnits );

  /* Destroy the structure itself, and exit. */
  LALFree( *grid );
  *grid = NULL;
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}


void
LALU8CreateGrid ( LALStatus *stat, UINT8Grid **grid, UINT4Vector *dimLength, UINT4 dimension )
{
  INITSTATUS( stat, "LALU8CreateGrid", GRIDC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( dimLength, stat, GRIDH_ENUL, GRIDH_MSGENUL );
  ASSERT( dimLength->data, stat, GRIDH_ENUL, GRIDH_MSGENUL );
  ASSERT( grid, stat, GRIDH_ENUL, GRIDH_MSGENUL );
  ASSERT( !*grid, stat, GRIDH_EOUT, GRIDH_MSGEOUT );
  ASSERT( dimension <= dimLength->length, stat, GRIDH_ENUL,
	  GRIDH_MSGENUL );

  /* Allocate the grid. */
  if ( !( *grid = ( UINT8Grid *)LALMalloc( sizeof( UINT8Grid ) ) ) ) {
    ABORT( stat, GRIDH_EMEM, GRIDH_MSGEMEM );
  }
  memset( *grid, 0, sizeof( UINT8Grid ) );

  /* Allocate the dimension units array. */
  if ( !( (*grid)->dimUnits = (LALUnit *)
	  LALMalloc( dimension*sizeof(LALUnit) ) ) ) {
    LALFree( *grid );
    *grid = NULL;
    ABORT( stat, GRIDH_EMEM, GRIDH_MSGEMEM );
  }
  memset( (*grid)->dimUnits, 0, dimension*sizeof(LALUnit) );

  /* Allocate the offset and interval vectors. */
  LALDCreateVector( stat->statusPtr, &((*grid)->offset), dimension );
  BEGINFAIL( stat ) {
    LALFree( (*grid)->dimUnits );
    LALFree( *grid );
    *grid = NULL;
  } ENDFAIL( stat );
  LALDCreateVector( stat->statusPtr, &((*grid)->interval), dimension );
  BEGINFAIL( stat ) {
    TRY( LALDDestroyVector( stat->statusPtr, &((*grid)->offset) ), stat );
    LALFree( (*grid)->dimUnits );
    LALFree( *grid );
    *grid = NULL;
  } ENDFAIL( stat );

  /* Allocate the data array. */
  LALU8CreateArray ( stat->statusPtr, &((*grid)->data), dimLength );
  BEGINFAIL( stat ) {
    TRY( LALDDestroyVector( stat->statusPtr, &((*grid)->interval) ), stat );
    TRY( LALDDestroyVector( stat->statusPtr, &((*grid)->offset) ), stat );
    LALFree( (*grid)->dimUnits );
    LALFree( *grid );
    *grid = NULL;
  } ENDFAIL( stat );

  /* Done. */
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}


void
LALU8DestroyGrid ( LALStatus *stat, UINT8Grid **grid )
{
  INITSTATUS( stat, "LALU8DestroyGrid", GRIDC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input argument. */
  ASSERT( grid, stat, GRIDH_ENUL, GRIDH_MSGENUL );
  ASSERT( *grid, stat, GRIDH_ENUL, GRIDH_MSGENUL );

  /* Destroy the internal arrays and vectors. */
  TRY( LALU8DestroyArray ( stat->statusPtr, &((*grid)->data) ), stat );
  TRY( LALDDestroyVector( stat->statusPtr, &((*grid)->interval) ), stat );
  TRY( LALDDestroyVector( stat->statusPtr, &((*grid)->offset) ), stat );
  LALFree( (*grid)->dimUnits );

  /* Destroy the structure itself, and exit. */
  LALFree( *grid );
  *grid = NULL;
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}


void
LALSCreateGrid ( LALStatus *stat, REAL4Grid **grid, UINT4Vector *dimLength, UINT4 dimension )
{
  INITSTATUS( stat, "LALSCreateGrid", GRIDC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( dimLength, stat, GRIDH_ENUL, GRIDH_MSGENUL );
  ASSERT( dimLength->data, stat, GRIDH_ENUL, GRIDH_MSGENUL );
  ASSERT( grid, stat, GRIDH_ENUL, GRIDH_MSGENUL );
  ASSERT( !*grid, stat, GRIDH_EOUT, GRIDH_MSGEOUT );
  ASSERT( dimension <= dimLength->length, stat, GRIDH_ENUL,
	  GRIDH_MSGENUL );

  /* Allocate the grid. */
  if ( !( *grid = ( REAL4Grid *)LALMalloc( sizeof( REAL4Grid ) ) ) ) {
    ABORT( stat, GRIDH_EMEM, GRIDH_MSGEMEM );
  }
  memset( *grid, 0, sizeof( REAL4Grid ) );

  /* Allocate the dimension units array. */
  if ( !( (*grid)->dimUnits = (LALUnit *)
	  LALMalloc( dimension*sizeof(LALUnit) ) ) ) {
    LALFree( *grid );
    *grid = NULL;
    ABORT( stat, GRIDH_EMEM, GRIDH_MSGEMEM );
  }
  memset( (*grid)->dimUnits, 0, dimension*sizeof(LALUnit) );

  /* Allocate the offset and interval vectors. */
  LALDCreateVector( stat->statusPtr, &((*grid)->offset), dimension );
  BEGINFAIL( stat ) {
    LALFree( (*grid)->dimUnits );
    LALFree( *grid );
    *grid = NULL;
  } ENDFAIL( stat );
  LALDCreateVector( stat->statusPtr, &((*grid)->interval), dimension );
  BEGINFAIL( stat ) {
    TRY( LALDDestroyVector( stat->statusPtr, &((*grid)->offset) ), stat );
    LALFree( (*grid)->dimUnits );
    LALFree( *grid );
    *grid = NULL;
  } ENDFAIL( stat );

  /* Allocate the data array. */
  LALSCreateArray ( stat->statusPtr, &((*grid)->data), dimLength );
  BEGINFAIL( stat ) {
    TRY( LALDDestroyVector( stat->statusPtr, &((*grid)->interval) ), stat );
    TRY( LALDDestroyVector( stat->statusPtr, &((*grid)->offset) ), stat );
    LALFree( (*grid)->dimUnits );
    LALFree( *grid );
    *grid = NULL;
  } ENDFAIL( stat );

  /* Done. */
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}


void
LALSDestroyGrid ( LALStatus *stat, REAL4Grid **grid )
{
  INITSTATUS( stat, "LALSDestroyGrid", GRIDC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input argument. */
  ASSERT( grid, stat, GRIDH_ENUL, GRIDH_MSGENUL );
  ASSERT( *grid, stat, GRIDH_ENUL, GRIDH_MSGENUL );

  /* Destroy the internal arrays and vectors. */
  TRY( LALSDestroyArray ( stat->statusPtr, &((*grid)->data) ), stat );
  TRY( LALDDestroyVector( stat->statusPtr, &((*grid)->interval) ), stat );
  TRY( LALDDestroyVector( stat->statusPtr, &((*grid)->offset) ), stat );
  LALFree( (*grid)->dimUnits );

  /* Destroy the structure itself, and exit. */
  LALFree( *grid );
  *grid = NULL;
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}


void
LALDCreateGrid ( LALStatus *stat, REAL8Grid **grid, UINT4Vector *dimLength, UINT4 dimension )
{
  INITSTATUS( stat, "LALDCreateGrid", GRIDC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( dimLength, stat, GRIDH_ENUL, GRIDH_MSGENUL );
  ASSERT( dimLength->data, stat, GRIDH_ENUL, GRIDH_MSGENUL );
  ASSERT( grid, stat, GRIDH_ENUL, GRIDH_MSGENUL );
  ASSERT( !*grid, stat, GRIDH_EOUT, GRIDH_MSGEOUT );
  ASSERT( dimension <= dimLength->length, stat, GRIDH_ENUL,
	  GRIDH_MSGENUL );

  /* Allocate the grid. */
  if ( !( *grid = ( REAL8Grid *)LALMalloc( sizeof( REAL8Grid ) ) ) ) {
    ABORT( stat, GRIDH_EMEM, GRIDH_MSGEMEM );
  }
  memset( *grid, 0, sizeof( REAL8Grid ) );

  /* Allocate the dimension units array. */
  if ( !( (*grid)->dimUnits = (LALUnit *)
	  LALMalloc( dimension*sizeof(LALUnit) ) ) ) {
    LALFree( *grid );
    *grid = NULL;
    ABORT( stat, GRIDH_EMEM, GRIDH_MSGEMEM );
  }
  memset( (*grid)->dimUnits, 0, dimension*sizeof(LALUnit) );

  /* Allocate the offset and interval vectors. */
  LALDCreateVector( stat->statusPtr, &((*grid)->offset), dimension );
  BEGINFAIL( stat ) {
    LALFree( (*grid)->dimUnits );
    LALFree( *grid );
    *grid = NULL;
  } ENDFAIL( stat );
  LALDCreateVector( stat->statusPtr, &((*grid)->interval), dimension );
  BEGINFAIL( stat ) {
    TRY( LALDDestroyVector( stat->statusPtr, &((*grid)->offset) ), stat );
    LALFree( (*grid)->dimUnits );
    LALFree( *grid );
    *grid = NULL;
  } ENDFAIL( stat );

  /* Allocate the data array. */
  LALDCreateArray ( stat->statusPtr, &((*grid)->data), dimLength );
  BEGINFAIL( stat ) {
    TRY( LALDDestroyVector( stat->statusPtr, &((*grid)->interval) ), stat );
    TRY( LALDDestroyVector( stat->statusPtr, &((*grid)->offset) ), stat );
    LALFree( (*grid)->dimUnits );
    LALFree( *grid );
    *grid = NULL;
  } ENDFAIL( stat );

  /* Done. */
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}


void
LALDDestroyGrid ( LALStatus *stat, REAL8Grid **grid )
{
  INITSTATUS( stat, "LALDDestroyGrid", GRIDC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input argument. */
  ASSERT( grid, stat, GRIDH_ENUL, GRIDH_MSGENUL );
  ASSERT( *grid, stat, GRIDH_ENUL, GRIDH_MSGENUL );

  /* Destroy the internal arrays and vectors. */
  TRY( LALDDestroyArray ( stat->statusPtr, &((*grid)->data) ), stat );
  TRY( LALDDestroyVector( stat->statusPtr, &((*grid)->interval) ), stat );
  TRY( LALDDestroyVector( stat->statusPtr, &((*grid)->offset) ), stat );
  LALFree( (*grid)->dimUnits );

  /* Destroy the structure itself, and exit. */
  LALFree( *grid );
  *grid = NULL;
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}


void
LALCCreateGrid ( LALStatus *stat, COMPLEX8Grid **grid, UINT4Vector *dimLength, UINT4 dimension )
{
  INITSTATUS( stat, "LALCCreateGrid", GRIDC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( dimLength, stat, GRIDH_ENUL, GRIDH_MSGENUL );
  ASSERT( dimLength->data, stat, GRIDH_ENUL, GRIDH_MSGENUL );
  ASSERT( grid, stat, GRIDH_ENUL, GRIDH_MSGENUL );
  ASSERT( !*grid, stat, GRIDH_EOUT, GRIDH_MSGEOUT );
  ASSERT( dimension <= dimLength->length, stat, GRIDH_ENUL,
	  GRIDH_MSGENUL );

  /* Allocate the grid. */
  if ( !( *grid = ( COMPLEX8Grid *)LALMalloc( sizeof( COMPLEX8Grid ) ) ) ) {
    ABORT( stat, GRIDH_EMEM, GRIDH_MSGEMEM );
  }
  memset( *grid, 0, sizeof( COMPLEX8Grid ) );

  /* Allocate the dimension units array. */
  if ( !( (*grid)->dimUnits = (LALUnit *)
	  LALMalloc( dimension*sizeof(LALUnit) ) ) ) {
    LALFree( *grid );
    *grid = NULL;
    ABORT( stat, GRIDH_EMEM, GRIDH_MSGEMEM );
  }
  memset( (*grid)->dimUnits, 0, dimension*sizeof(LALUnit) );

  /* Allocate the offset and interval vectors. */
  LALDCreateVector( stat->statusPtr, &((*grid)->offset), dimension );
  BEGINFAIL( stat ) {
    LALFree( (*grid)->dimUnits );
    LALFree( *grid );
    *grid = NULL;
  } ENDFAIL( stat );
  LALDCreateVector( stat->statusPtr, &((*grid)->interval), dimension );
  BEGINFAIL( stat ) {
    TRY( LALDDestroyVector( stat->statusPtr, &((*grid)->offset) ), stat );
    LALFree( (*grid)->dimUnits );
    LALFree( *grid );
    *grid = NULL;
  } ENDFAIL( stat );

  /* Allocate the data array. */
  LALCCreateArray ( stat->statusPtr, &((*grid)->data), dimLength );
  BEGINFAIL( stat ) {
    TRY( LALDDestroyVector( stat->statusPtr, &((*grid)->interval) ), stat );
    TRY( LALDDestroyVector( stat->statusPtr, &((*grid)->offset) ), stat );
    LALFree( (*grid)->dimUnits );
    LALFree( *grid );
    *grid = NULL;
  } ENDFAIL( stat );

  /* Done. */
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}


void
LALCDestroyGrid ( LALStatus *stat, COMPLEX8Grid **grid )
{
  INITSTATUS( stat, "LALCDestroyGrid", GRIDC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input argument. */
  ASSERT( grid, stat, GRIDH_ENUL, GRIDH_MSGENUL );
  ASSERT( *grid, stat, GRIDH_ENUL, GRIDH_MSGENUL );

  /* Destroy the internal arrays and vectors. */
  TRY( LALCDestroyArray ( stat->statusPtr, &((*grid)->data) ), stat );
  TRY( LALDDestroyVector( stat->statusPtr, &((*grid)->interval) ), stat );
  TRY( LALDDestroyVector( stat->statusPtr, &((*grid)->offset) ), stat );
  LALFree( (*grid)->dimUnits );

  /* Destroy the structure itself, and exit. */
  LALFree( *grid );
  *grid = NULL;
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}


void
LALZCreateGrid ( LALStatus *stat, COMPLEX16Grid **grid, UINT4Vector *dimLength, UINT4 dimension )
{
  INITSTATUS( stat, "LALZCreateGrid", GRIDC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input arguments. */
  ASSERT( dimLength, stat, GRIDH_ENUL, GRIDH_MSGENUL );
  ASSERT( dimLength->data, stat, GRIDH_ENUL, GRIDH_MSGENUL );
  ASSERT( grid, stat, GRIDH_ENUL, GRIDH_MSGENUL );
  ASSERT( !*grid, stat, GRIDH_EOUT, GRIDH_MSGEOUT );
  ASSERT( dimension <= dimLength->length, stat, GRIDH_ENUL,
	  GRIDH_MSGENUL );

  /* Allocate the grid. */
  if ( !( *grid = ( COMPLEX16Grid *)LALMalloc( sizeof( COMPLEX16Grid ) ) ) ) {
    ABORT( stat, GRIDH_EMEM, GRIDH_MSGEMEM );
  }
  memset( *grid, 0, sizeof( COMPLEX16Grid ) );

  /* Allocate the dimension units array. */
  if ( !( (*grid)->dimUnits = (LALUnit *)
	  LALMalloc( dimension*sizeof(LALUnit) ) ) ) {
    LALFree( *grid );
    *grid = NULL;
    ABORT( stat, GRIDH_EMEM, GRIDH_MSGEMEM );
  }
  memset( (*grid)->dimUnits, 0, dimension*sizeof(LALUnit) );

  /* Allocate the offset and interval vectors. */
  LALDCreateVector( stat->statusPtr, &((*grid)->offset), dimension );
  BEGINFAIL( stat ) {
    LALFree( (*grid)->dimUnits );
    LALFree( *grid );
    *grid = NULL;
  } ENDFAIL( stat );
  LALDCreateVector( stat->statusPtr, &((*grid)->interval), dimension );
  BEGINFAIL( stat ) {
    TRY( LALDDestroyVector( stat->statusPtr, &((*grid)->offset) ), stat );
    LALFree( (*grid)->dimUnits );
    LALFree( *grid );
    *grid = NULL;
  } ENDFAIL( stat );

  /* Allocate the data array. */
  LALZCreateArray ( stat->statusPtr, &((*grid)->data), dimLength );
  BEGINFAIL( stat ) {
    TRY( LALDDestroyVector( stat->statusPtr, &((*grid)->interval) ), stat );
    TRY( LALDDestroyVector( stat->statusPtr, &((*grid)->offset) ), stat );
    LALFree( (*grid)->dimUnits );
    LALFree( *grid );
    *grid = NULL;
  } ENDFAIL( stat );

  /* Done. */
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}


void
LALZDestroyGrid ( LALStatus *stat, COMPLEX16Grid **grid )
{
  INITSTATUS( stat, "LALZDestroyGrid", GRIDC );
  ATTATCHSTATUSPTR( stat );

  /* Check for valid input argument. */
  ASSERT( grid, stat, GRIDH_ENUL, GRIDH_MSGENUL );
  ASSERT( *grid, stat, GRIDH_ENUL, GRIDH_MSGENUL );

  /* Destroy the internal arrays and vectors. */
  TRY( LALZDestroyArray ( stat->statusPtr, &((*grid)->data) ), stat );
  TRY( LALDDestroyVector( stat->statusPtr, &((*grid)->interval) ), stat );
  TRY( LALDDestroyVector( stat->statusPtr, &((*grid)->offset) ), stat );
  LALFree( (*grid)->dimUnits );

  /* Destroy the structure itself, and exit. */
  LALFree( *grid );
  *grid = NULL;
  DETATCHSTATUSPTR( stat );
  RETURN( stat );
}
