/************************************ <lalVerbatim file="PrintTimeSeriesCV">
Author: Whelan, J. T.
$Id$
************************************* </lalVerbatim> */

/* <lalLaTeX>

\subsection{Module \texttt{PrintTimeSeries.c}}
\label{ss:PrintTimeSeries.c}

Print a $\langle\mbox{datatype}\rangle$TimeSeries object into a
file.  For use in non-production and test code only.

\subsubsection*{Prototypes}
\vspace{0.1in}
\input{PrintTimeSeriesCP}
\idx{LALZPrintTimeSeries()}
\idx{LALCPrintTimeSeries()}
\idx{LALDPrintTimeSeries()}
\idx{LALSPrintTimeSeries()}
\idx{LALI2PrintTimeSeries()}
\idx{LALI4PrintTimeSeries()}
\idx{LALI8PrintTimeSeries()}
\idx{LALU2PrintTimeSeries()}
\idx{LALU4PrintTimeSeries()}
\idx{LALU8PrintTimeSeries()}
\idx{LALPrintTimeSeries()}

\subsubsection*{Description}

Each member of this family of functions prints the elements of
$\langle\mbox{datatype}\rangle$\verb+TimeSeries+ into a file.  Note:
the file name is specified using a character string.  This function is
for debugging use only: its arguments do not conform to LAL standards
so it should not be used in any real analysis codes.

\subsubsection*{Algorithm}

\subsubsection*{Uses}

\begin{verbatim}
LALFopen()
LALFclose()
LALCHARCreateVector()
LALCHARDestroyVector()
LALUnitAsString()
\end{verbatim}

\subsubsection*{Notes}

This function's arguments do not conform to the LAL spec.  For this
reason it should only be used for debugging purposes in test
functions, not in any production code.

Additionally, since printf cannot handle INT8 as integers, the
functions \verb&LALI8PrintTimeSeries()& and
\verb&LALU8PrintTimeSeries()& use a typecast to REAL8 and are thus
only valid for numbers between around $-10^{15}$ and $10^{15}$.

The first five lines of the file are a header containing:
\begin{enumerate}
\item the name of the series
\item the starting epoch 
\item the units expressed in terms of the basic SI units
\item column labels
\end{enumerate}
after which come the data, one per line.


The output format is two or three tab-separated columns: the first
column is the time corresponding to the row in question, in seconds
after the series' starting epoch; for real and integer time
series, the second column is the value of the series; for complex time
series, the second column is the real part and the third the imaginary
part of the value.

\vfill{\footnotesize\input{PrintTimeSeriesCV}}

</lalLaTeX> */


#include <lal/LALStdlib.h>
#include <lal/LALStdio.h>
#include <lal/LALDatatypes.h>
#include <lal/PrintFTSeries.h>

void LALCHARCreateVector( LALStatus *, CHARVector **, UINT4 );
void LALCHARDestroyVector( LALStatus *, CHARVector ** );
void LALUnitAsString( LALStatus *status, CHARVector *output,
                      const LALUnit *input );

enum { LALUnitTextSize = sizeof("10^-32768 m^-32768/32767 kg^-32768/32767 "
				"s^-32768/32767 A^-32768/32767 " 
				"K^-32768/32767 strain^-32768/32767 "
				"count^-32768/32767") };

/* <lalVerbatim file="PrintTimeSeriesNRCSID"> */
NRCSID( PRINTTIMESERIESC, "$Id: " );
/* </lalVerbatim> */

































/* <lalVerbatim file="PrintTimeSeriesCP"> */
void LALZPrintTimeSeries ( COMPLEX16TimeSeries *series, const CHAR *filename ) 
{ /* </lalVerbatim> */
  REAL8 t;
  COMPLEX16 *data;
  FILE *fp;
  UINT4 i;
  static LALStatus status;
  CHARVector *unitString;

  if (series==NULL) return;

  /* *(series->data) is a COMPLEX16Sequence */
  /* series->data->data is a pointer to COMPLEX16 */

  /* Make a COMPLEX16 pointer which points to the first memory address not
   * belonging to the sequence
   */

  /* open output file */
  fp=LALFopen(filename,"w");
  fprintf(fp,"# %s\n",series->name);
  if (series->f0) {
     fprintf(fp,"# Heterodyned at %g Hz\n",series->f0);
  }
  else {
    fprintf(fp,"# \n");
  }
  fprintf(fp,"# Epoch is %d seconds, %d nanoseconds\n",
          series->epoch.gpsSeconds,series->epoch.gpsNanoSeconds);
  unitString = NULL;
  LALCHARCreateVector(&status, &unitString, LALUnitTextSize);
  LALUnitAsString(&status, unitString, &(series->sampleUnits));
  fprintf(fp,"# Units are (%s)\n",unitString->data);
  fprintf(fp,"# Seconds since epoch\tRe(Value)\tIm(Value)\n");
  LALCHARDestroyVector(&status, &unitString);
  for ( i = 0; i < series->data->length; ++i )
  {
    t = i * series->deltaT;
    data = &(series->data->data[i]);
    fprintf(fp,"%e\t%le\t%le\n",t,data->re,data->im);
  }	

  LALFclose(fp);

  return;
}


































/* <lalVerbatim file="PrintTimeSeriesCP"> */
void LALCPrintTimeSeries ( COMPLEX8TimeSeries *series, const CHAR *filename ) 
{ /* </lalVerbatim> */
  REAL8 t;
  COMPLEX8 *data;
  FILE *fp;
  UINT4 i;
  static LALStatus status;
  CHARVector *unitString;

  if (series==NULL) return;

  /* *(series->data) is a COMPLEX8Sequence */
  /* series->data->data is a pointer to COMPLEX8 */

  /* Make a COMPLEX8 pointer which points to the first memory address not
   * belonging to the sequence
   */

  /* open output file */
  fp=LALFopen(filename,"w");
  fprintf(fp,"# %s\n",series->name);
  if (series->f0) {
     fprintf(fp,"# Heterodyned at %g Hz\n",series->f0);
  }
  else {
    fprintf(fp,"# \n");
  }
  fprintf(fp,"# Epoch is %d seconds, %d nanoseconds\n",
          series->epoch.gpsSeconds,series->epoch.gpsNanoSeconds);
  unitString = NULL;
  LALCHARCreateVector(&status, &unitString, LALUnitTextSize);
  LALUnitAsString(&status, unitString, &(series->sampleUnits));
  fprintf(fp,"# Units are (%s)\n",unitString->data);
  fprintf(fp,"# Seconds since epoch\tRe(Value)\tIm(Value)\n");
  LALCHARDestroyVector(&status, &unitString);
  for ( i = 0; i < series->data->length; ++i )
  {
    t = i * series->deltaT;
    data = &(series->data->data[i]);
    fprintf(fp,"%e\t%e\t%e\n",t,data->re,data->im);
  }	

  LALFclose(fp);

  return;
}


































/* <lalVerbatim file="PrintTimeSeriesCP"> */
void LALDPrintTimeSeries ( REAL8TimeSeries *series, const CHAR *filename ) 
{ /* </lalVerbatim> */
  REAL8 t;
  REAL8 *data;
  FILE *fp;
  UINT4 i;
  static LALStatus status;
  CHARVector *unitString;

  if (series==NULL) return;

  /* *(series->data) is a REAL8Sequence */
  /* series->data->data is a pointer to REAL8 */

  /* Make a REAL8 pointer which points to the first memory address not
   * belonging to the sequence
   */

  /* open output file */
  fp=LALFopen(filename,"w");
  fprintf(fp,"# %s\n",series->name);
  if (series->f0) {
     fprintf(fp,"# Heterodyned at %g Hz\n",series->f0);
  }
  else {
    fprintf(fp,"# \n");
  }
  fprintf(fp,"# Epoch is %d seconds, %d nanoseconds\n",
          series->epoch.gpsSeconds,series->epoch.gpsNanoSeconds);
  unitString = NULL;
  LALCHARCreateVector(&status, &unitString, LALUnitTextSize);
  LALUnitAsString(&status, unitString, &(series->sampleUnits));
  fprintf(fp,"# Units are (%s)\n",unitString->data);
  fprintf(fp,"# Seconds since epoch\tValue\n");
  LALCHARDestroyVector(&status, &unitString);
  for ( i = 0; i < series->data->length; ++i )
  {
    t = i * series->deltaT;
    data = &(series->data->data[i]);
    fprintf(fp,"%e\t%le\n",t,*data);
  }	

  LALFclose(fp);

  return;
}


































/* <lalVerbatim file="PrintTimeSeriesCP"> */
void LALSPrintTimeSeries ( REAL4TimeSeries *series, const CHAR *filename ) 
{ /* </lalVerbatim> */
  REAL8 t;
  REAL4 *data;
  FILE *fp;
  UINT4 i;
  static LALStatus status;
  CHARVector *unitString;

  if (series==NULL) return;

  /* *(series->data) is a REAL4Sequence */
  /* series->data->data is a pointer to REAL4 */

  /* Make a REAL4 pointer which points to the first memory address not
   * belonging to the sequence
   */

  /* open output file */
  fp=LALFopen(filename,"w");
  fprintf(fp,"# %s\n",series->name);
  if (series->f0) {
     fprintf(fp,"# Heterodyned at %g Hz\n",series->f0);
  }
  else {
    fprintf(fp,"# \n");
  }
  fprintf(fp,"# Epoch is %d seconds, %d nanoseconds\n",
          series->epoch.gpsSeconds,series->epoch.gpsNanoSeconds);
  unitString = NULL;
  LALCHARCreateVector(&status, &unitString, LALUnitTextSize);
  LALUnitAsString(&status, unitString, &(series->sampleUnits));
  fprintf(fp,"# Units are (%s)\n",unitString->data);
  fprintf(fp,"# Seconds since epoch\tValue\n");
  LALCHARDestroyVector(&status, &unitString);
  for ( i = 0; i < series->data->length; ++i )
  {
    t = i * series->deltaT;
    data = &(series->data->data[i]);
    fprintf(fp,"%e\t%e\n",t,*data);
  }	

  LALFclose(fp);

  return;
}


































/* <lalVerbatim file="PrintTimeSeriesCP"> */
void LALI2PrintTimeSeries ( INT2TimeSeries *series, const CHAR *filename ) 
{ /* </lalVerbatim> */
  REAL8 t;
  INT2 *data;
  FILE *fp;
  UINT4 i;
  static LALStatus status;
  CHARVector *unitString;

  if (series==NULL) return;

  /* *(series->data) is a INT2Sequence */
  /* series->data->data is a pointer to INT2 */

  /* Make a INT2 pointer which points to the first memory address not
   * belonging to the sequence
   */

  /* open output file */
  fp=LALFopen(filename,"w");
  fprintf(fp,"# %s\n",series->name);
  if (series->f0) {
     fprintf(fp,"# Heterodyned at %g Hz\n",series->f0);
  }
  else {
    fprintf(fp,"# \n");
  }
  fprintf(fp,"# Epoch is %d seconds, %d nanoseconds\n",
          series->epoch.gpsSeconds,series->epoch.gpsNanoSeconds);
  unitString = NULL;
  LALCHARCreateVector(&status, &unitString, LALUnitTextSize);
  LALUnitAsString(&status, unitString, &(series->sampleUnits));
  fprintf(fp,"# Units are (%s)\n",unitString->data);
  fprintf(fp,"# Seconds since epoch\tValue\n");
  LALCHARDestroyVector(&status, &unitString);
  for ( i = 0; i < series->data->length; ++i )
  {
    t = i * series->deltaT;
    data = &(series->data->data[i]);
    fprintf(fp,"%g\t%i\n",t,*data);
  }	

  LALFclose(fp);

  return;
}


































/* <lalVerbatim file="PrintTimeSeriesCP"> */
void LALI4PrintTimeSeries ( INT4TimeSeries *series, const CHAR *filename ) 
{ /* </lalVerbatim> */
  REAL8 t;
  INT4 *data;
  FILE *fp;
  UINT4 i;
  static LALStatus status;
  CHARVector *unitString;

  if (series==NULL) return;

  /* *(series->data) is a INT4Sequence */
  /* series->data->data is a pointer to INT4 */

  /* Make a INT4 pointer which points to the first memory address not
   * belonging to the sequence
   */

  /* open output file */
  fp=LALFopen(filename,"w");
  fprintf(fp,"# %s\n",series->name);
  if (series->f0) {
     fprintf(fp,"# Heterodyned at %g Hz\n",series->f0);
  }
  else {
    fprintf(fp,"# \n");
  }
  fprintf(fp,"# Epoch is %d seconds, %d nanoseconds\n",
          series->epoch.gpsSeconds,series->epoch.gpsNanoSeconds);
  unitString = NULL;
  LALCHARCreateVector(&status, &unitString, LALUnitTextSize);
  LALUnitAsString(&status, unitString, &(series->sampleUnits));
  fprintf(fp,"# Units are (%s)\n",unitString->data);
  fprintf(fp,"# Seconds since epoch\tValue\n");
  LALCHARDestroyVector(&status, &unitString);
  for ( i = 0; i < series->data->length; ++i )
  {
    t = i * series->deltaT;
    data = &(series->data->data[i]);
    fprintf(fp,"%g\t%i\n",t,*data);
  }	

  LALFclose(fp);

  return;
}


































/* <lalVerbatim file="PrintTimeSeriesCP"> */
void LALI8PrintTimeSeries ( INT8TimeSeries *series, const CHAR *filename ) 
{ /* </lalVerbatim> */
  REAL8 t;
  INT8 *data;
  FILE *fp;
  UINT4 i;
  static LALStatus status;
  CHARVector *unitString;

  if (series==NULL) return;

  /* *(series->data) is a INT8Sequence */
  /* series->data->data is a pointer to INT8 */

  /* Make a INT8 pointer which points to the first memory address not
   * belonging to the sequence
   */

  /* open output file */
  fp=LALFopen(filename,"w");
  fprintf(fp,"# %s\n",series->name);
  if (series->f0) {
     fprintf(fp,"# Heterodyned at %g Hz\n",series->f0);
  }
  else {
    fprintf(fp,"# \n");
  }
  fprintf(fp,"# Epoch is %d seconds, %d nanoseconds\n",
          series->epoch.gpsSeconds,series->epoch.gpsNanoSeconds);
  unitString = NULL;
  LALCHARCreateVector(&status, &unitString, LALUnitTextSize);
  LALUnitAsString(&status, unitString, &(series->sampleUnits));
  fprintf(fp,"# Units are (%s)\n",unitString->data);
  fprintf(fp,"# Seconds since epoch\tValue\n");
  LALCHARDestroyVector(&status, &unitString);
  for ( i = 0; i < series->data->length; ++i )
  {
    t = i * series->deltaT;
    data = &(series->data->data[i]);
    fprintf(fp,"%g\t%0.0f\n",t,(REAL8) *data);
  }	

  LALFclose(fp);

  return;
}

/* Note that LALI8PrintTimeSeries does a typecast to REAL8 and is thus
 * inaccurate for numbers >~ 1e15 
 */

































/* <lalVerbatim file="PrintTimeSeriesCP"> */
void LALU2PrintTimeSeries ( UINT2TimeSeries *series, const CHAR *filename ) 
{ /* </lalVerbatim> */
  REAL8 t;
  UINT2 *data;
  FILE *fp;
  UINT4 i;
  static LALStatus status;
  CHARVector *unitString;

  if (series==NULL) return;

  /* *(series->data) is a UINT2Sequence */
  /* series->data->data is a pointer to UINT2 */

  /* Make a UINT2 pointer which points to the first memory address not
   * belonging to the sequence
   */

  /* open output file */
  fp=LALFopen(filename,"w");
  fprintf(fp,"# %s\n",series->name);
  if (series->f0) {
     fprintf(fp,"# Heterodyned at %g Hz\n",series->f0);
  }
  else {
    fprintf(fp,"# \n");
  }
  fprintf(fp,"# Epoch is %d seconds, %d nanoseconds\n",
          series->epoch.gpsSeconds,series->epoch.gpsNanoSeconds);
  unitString = NULL;
  LALCHARCreateVector(&status, &unitString, LALUnitTextSize);
  LALUnitAsString(&status, unitString, &(series->sampleUnits));
  fprintf(fp,"# Units are (%s)\n",unitString->data);
  fprintf(fp,"# Seconds since epoch\tValue\n");
  LALCHARDestroyVector(&status, &unitString);
  for ( i = 0; i < series->data->length; ++i )
  {
    t = i * series->deltaT;
    data = &(series->data->data[i]);
    fprintf(fp,"%g\t%i\n",t,*data);
  }	

  LALFclose(fp);

  return;
}


































/* <lalVerbatim file="PrintTimeSeriesCP"> */
void LALU4PrintTimeSeries ( UINT4TimeSeries *series, const CHAR *filename ) 
{ /* </lalVerbatim> */
  REAL8 t;
  UINT4 *data;
  FILE *fp;
  UINT4 i;
  static LALStatus status;
  CHARVector *unitString;

  if (series==NULL) return;

  /* *(series->data) is a UINT4Sequence */
  /* series->data->data is a pointer to UINT4 */

  /* Make a UINT4 pointer which points to the first memory address not
   * belonging to the sequence
   */

  /* open output file */
  fp=LALFopen(filename,"w");
  fprintf(fp,"# %s\n",series->name);
  if (series->f0) {
     fprintf(fp,"# Heterodyned at %g Hz\n",series->f0);
  }
  else {
    fprintf(fp,"# \n");
  }
  fprintf(fp,"# Epoch is %d seconds, %d nanoseconds\n",
          series->epoch.gpsSeconds,series->epoch.gpsNanoSeconds);
  unitString = NULL;
  LALCHARCreateVector(&status, &unitString, LALUnitTextSize);
  LALUnitAsString(&status, unitString, &(series->sampleUnits));
  fprintf(fp,"# Units are (%s)\n",unitString->data);
  fprintf(fp,"# Seconds since epoch\tValue\n");
  LALCHARDestroyVector(&status, &unitString);
  for ( i = 0; i < series->data->length; ++i )
  {
    t = i * series->deltaT;
    data = &(series->data->data[i]);
    fprintf(fp,"%g\t%i\n",t,*data);
  }	

  LALFclose(fp);

  return;
}


































/* <lalVerbatim file="PrintTimeSeriesCP"> */
void LALU8PrintTimeSeries ( UINT8TimeSeries *series, const CHAR *filename ) 
{ /* </lalVerbatim> */
  REAL8 t;
  UINT8 *data;
  FILE *fp;
  UINT4 i;
  static LALStatus status;
  CHARVector *unitString;

  if (series==NULL) return;

  /* *(series->data) is a UINT8Sequence */
  /* series->data->data is a pointer to UINT8 */

  /* Make a UINT8 pointer which points to the first memory address not
   * belonging to the sequence
   */

  /* open output file */
  fp=LALFopen(filename,"w");
  fprintf(fp,"# %s\n",series->name);
  if (series->f0) {
     fprintf(fp,"# Heterodyned at %g Hz\n",series->f0);
  }
  else {
    fprintf(fp,"# \n");
  }
  fprintf(fp,"# Epoch is %d seconds, %d nanoseconds\n",
          series->epoch.gpsSeconds,series->epoch.gpsNanoSeconds);
  unitString = NULL;
  LALCHARCreateVector(&status, &unitString, LALUnitTextSize);
  LALUnitAsString(&status, unitString, &(series->sampleUnits));
  fprintf(fp,"# Units are (%s)\n",unitString->data);
  fprintf(fp,"# Seconds since epoch\tValue\n");
  LALCHARDestroyVector(&status, &unitString);
  for ( i = 0; i < series->data->length; ++i )
  {
    t = i * series->deltaT;
    data = &(series->data->data[i]);
    fprintf(fp,"%g\t%0.0f\n",t,(REAL8) *data);
  }	

  LALFclose(fp);

  return;
}

/* Note that LALU8PrintTimeSeries does a typecast to REAL8 and is thus
 * inaccurate for numbers >~ 1e15 
 */

































/* <lalVerbatim file="PrintTimeSeriesCP"> */
void LALPrintTimeSeries ( REAL4TimeSeries *series, const CHAR *filename ) 
{ /* </lalVerbatim> */
  REAL8 t;
  REAL4 *data;
  FILE *fp;
  UINT4 i;
  static LALStatus status;
  CHARVector *unitString;

  if (series==NULL) return;

  /* *(series->data) is a REAL4Sequence */
  /* series->data->data is a pointer to REAL4 */

  /* Make a REAL4 pointer which points to the first memory address not
   * belonging to the sequence
   */

  /* open output file */
  fp=LALFopen(filename,"w");
  fprintf(fp,"# %s\n",series->name);
  if (series->f0) {
     fprintf(fp,"# Heterodyned at %g Hz\n",series->f0);
  }
  else {
    fprintf(fp,"# \n");
  }
  fprintf(fp,"# Epoch is %d seconds, %d nanoseconds\n",
          series->epoch.gpsSeconds,series->epoch.gpsNanoSeconds);
  unitString = NULL;
  LALCHARCreateVector(&status, &unitString, LALUnitTextSize);
  LALUnitAsString(&status, unitString, &(series->sampleUnits));
  fprintf(fp,"# Units are (%s)\n",unitString->data);
  fprintf(fp,"# Seconds since epoch\tValue\n");
  LALCHARDestroyVector(&status, &unitString);
  for ( i = 0; i < series->data->length; ++i )
  {
    t = i * series->deltaT;
    data = &(series->data->data[i]);
    fprintf(fp,"%g\t%f\n",t,*data);
  }	

  LALFclose(fp);

  return;
}














