/************************************ <lalVerbatim file="PrintVectorCV">
Author: Allen, B.; generalized by J.T. Whelan
$Id$
************************************* </lalVerbatim> */

/* <lalLaTeX>

\subsection{Module \texttt{PrintVector.c}}
\label{ss:PrintVector.c}

Print a $\langle\mbox{datatype}\rangle$Vector object into a file.  For
use in non-production and test code only.

\subsubsection*{Prototypes}
\vspace{0.1in}
\input{PrintVectorCP}
\idx{LALZPrintVector()}
\idx{LALCPrintVector()}
\idx{LALDPrintVector()}
\idx{LALSPrintVector()}
\idx{LALI2PrintVector()}
\idx{LALI4PrintVector()}
\idx{LALI8PrintVector()}
\idx{LALU2PrintVector()}
\idx{LALU4PrintVector()}
\idx{LALU8PrintVector()}
\idx{LALCHARPrintVector()}
\idx{LALPrintVector()}

\subsubsection*{Description}

Each member of this family of functions prints the elements of
$\langle\mbox{datatype}\rangle$\verb+Vector+ into a file.  Note: the
file names are $\langle\mbox{DT}\rangle$\verb+PrintVector.000+,
$\langle\mbox{DT}\rangle$\verb+PrintVector.001+, and so on.
($\langle\mbox{DT}\rangle$ is the abbreviation for the datatype,
included in the function names above.) The file numbers are
incremented with each additional call.  This function is for debugging
use only: it uses a static internal variable to keep track of the file
number so it should not be used in any real analysis codes.

\subsubsection*{Algorithm}

\subsubsection*{Uses}

\begin{verbatim}
LALFopen()
LALFclose()
\end{verbatim}

\subsubsection*{Notes}

This function uses an internal static variable to keep track of file
numbers.  For this reason it should only be used for debugging
purposes in test functions, not in any production code.

Additionally, since printf cannot handle INT8 as integers, the
functions \verb&LALI8PrintVector()& and \verb&LALU8PrintVector()& use
a typecast to REAL8 and are thus only valid for numbers between around
$-10^{15}$ and $10^{15}$.

The output format is two or three space-separated columns: the first
column is the index of the element; the second is the element itself
for real and integer vectors and the real part of the element for
complex vectors; complex vectors have a third column containing the
imaginary part of the element.

\vfill{\footnotesize\input{PrintVectorCV}}

</lalLaTeX> */


#include <lal/LALStdlib.h>
#include <lal/LALStdio.h>
#include <lal/LALDatatypes.h>
#include <lal/PrintVector.h>

/* <lalVerbatim file="VectorFactoriesNRCSID"> */
NRCSID( PRINTVECTORC, "$Id$" );
/* </lalVerbatim> */































/* <lalVerbatim file="PrintVectorCP"> */
void LALZPrintVector ( COMPLEX16Vector *vector ) 
{ /* </lalVerbatim> */
  int i;
  static int filenum=0;
  FILE *fp;
  char fname[256];


  if (vector==NULL) return;

  /* open output file */
  sprintf(fname,"Z" "PrintVector.%03d",filenum++);
  fp=LALFopen(fname,"w");

  for (i=0;i<(int)vector->length;i++)
    fprintf(fp,"%i %g %g\n",i,vector->data[i].re,vector->data[i].im);

  LALFclose(fp);

  return;
}
































/* <lalVerbatim file="PrintVectorCP"> */
void LALCPrintVector ( COMPLEX8Vector *vector ) 
{ /* </lalVerbatim> */
  int i;
  static int filenum=0;
  FILE *fp;
  char fname[256];


  if (vector==NULL) return;

  /* open output file */
  sprintf(fname,"C" "PrintVector.%03d",filenum++);
  fp=LALFopen(fname,"w");

  for (i=0;i<(int)vector->length;i++)
    fprintf(fp,"%i %g %g\n",i,vector->data[i].re,vector->data[i].im);

  LALFclose(fp);

  return;
}
































/* <lalVerbatim file="PrintVectorCP"> */
void LALDPrintVector ( REAL8Vector *vector ) 
{ /* </lalVerbatim> */
  int i;
  static int filenum=0;
  FILE *fp;
  char fname[256];


  if (vector==NULL) return;

  /* open output file */
  sprintf(fname,"D" "PrintVector.%03d",filenum++);
  fp=LALFopen(fname,"w");

  for (i=0;i<(int)vector->length;i++)
    fprintf(fp,"%i %g\n",i,vector->data[i]);

  LALFclose(fp);

  return;
}
































/* <lalVerbatim file="PrintVectorCP"> */
void LALSPrintVector ( REAL4Vector *vector ) 
{ /* </lalVerbatim> */
  int i;
  static int filenum=0;
  FILE *fp;
  char fname[256];


  if (vector==NULL) return;

  /* open output file */
  sprintf(fname,"S" "PrintVector.%03d",filenum++);
  fp=LALFopen(fname,"w");

  for (i=0;i<(int)vector->length;i++)
    fprintf(fp,"%i %g\n",i,vector->data[i]);

  LALFclose(fp);

  return;
}
































/* <lalVerbatim file="PrintVectorCP"> */
void LALI2PrintVector ( INT2Vector *vector ) 
{ /* </lalVerbatim> */
  int i;
  static int filenum=0;
  FILE *fp;
  char fname[256];


  if (vector==NULL) return;

  /* open output file */
  sprintf(fname,"I2" "PrintVector.%03d",filenum++);
  fp=LALFopen(fname,"w");

  for (i=0;i<(int)vector->length;i++)
    fprintf(fp,"%i %i\n",i,vector->data[i]);

  LALFclose(fp);

  return;
}
































/* <lalVerbatim file="PrintVectorCP"> */
void LALI4PrintVector ( INT4Vector *vector ) 
{ /* </lalVerbatim> */
  int i;
  static int filenum=0;
  FILE *fp;
  char fname[256];


  if (vector==NULL) return;

  /* open output file */
  sprintf(fname,"I4" "PrintVector.%03d",filenum++);
  fp=LALFopen(fname,"w");

  for (i=0;i<(int)vector->length;i++)
    fprintf(fp,"%i %i\n",i,vector->data[i]);

  LALFclose(fp);

  return;
}
































/* <lalVerbatim file="PrintVectorCP"> */
void LALI8PrintVector ( INT8Vector *vector ) 
{ /* </lalVerbatim> */
  int i;
  static int filenum=0;
  FILE *fp;
  char fname[256];


  if (vector==NULL) return;

  /* open output file */
  sprintf(fname,"I8" "PrintVector.%03d",filenum++);
  fp=LALFopen(fname,"w");

  for (i=0;i<(int)vector->length;i++)
    fprintf(fp,"%i %0.0f\n",i,(REAL8) vector->data[i]);

  LALFclose(fp);

  return;
}

/* Note that LALI8PrintVector does a typecast to REAL8 and is thus
 * inaccurate for numbers >~ 1e15 
 */































/* <lalVerbatim file="PrintVectorCP"> */
void LALU2PrintVector ( UINT2Vector *vector ) 
{ /* </lalVerbatim> */
  int i;
  static int filenum=0;
  FILE *fp;
  char fname[256];


  if (vector==NULL) return;

  /* open output file */
  sprintf(fname,"U2" "PrintVector.%03d",filenum++);
  fp=LALFopen(fname,"w");

  for (i=0;i<(int)vector->length;i++)
    fprintf(fp,"%i %i\n",i,vector->data[i]);

  LALFclose(fp);

  return;
}
































/* <lalVerbatim file="PrintVectorCP"> */
void LALU4PrintVector ( UINT4Vector *vector ) 
{ /* </lalVerbatim> */
  int i;
  static int filenum=0;
  FILE *fp;
  char fname[256];


  if (vector==NULL) return;

  /* open output file */
  sprintf(fname,"U4" "PrintVector.%03d",filenum++);
  fp=LALFopen(fname,"w");

  for (i=0;i<(int)vector->length;i++)
    fprintf(fp,"%i %i\n",i,vector->data[i]);

  LALFclose(fp);

  return;
}
































/* <lalVerbatim file="PrintVectorCP"> */
void LALU8PrintVector ( UINT8Vector *vector ) 
{ /* </lalVerbatim> */
  int i;
  static int filenum=0;
  FILE *fp;
  char fname[256];


  if (vector==NULL) return;

  /* open output file */
  sprintf(fname,"U8" "PrintVector.%03d",filenum++);
  fp=LALFopen(fname,"w");

  for (i=0;i<(int)vector->length;i++)
    fprintf(fp,"%i %0.0f\n",i,(REAL8) vector->data[i]);

  LALFclose(fp);

  return;
}

/* Note that LALU8PrintVector does a typecast to REAL8 and is thus
 * inaccurate for numbers >~ 1e15 
 */































/* <lalVerbatim file="PrintVectorCP"> */
void LALCHARPrintVector ( CHARVector *vector ) 
{ /* </lalVerbatim> */
  int i;
  static int filenum=0;
  FILE *fp;
  char fname[256];


  if (vector==NULL) return;

  /* open output file */
  sprintf(fname,"CHAR" "PrintVector.%03d",filenum++);
  fp=LALFopen(fname,"w");

  for (i=0;i<(int)vector->length;i++)
    fprintf(fp,"%i %c\n",i,vector->data[i]);

  LALFclose(fp);

  return;
}
































/* <lalVerbatim file="PrintVectorCP"> */
void LALPrintVector ( REAL4Vector *vector ) 
{ /* </lalVerbatim> */
  int i;
  static int filenum=0;
  FILE *fp;
  char fname[256];


  if (vector==NULL) return;

  /* open output file */
  sprintf(fname,"" "PrintVector.%03d",filenum++);
  fp=LALFopen(fname,"w");

  for (i=0;i<(int)vector->length;i++)
    fprintf(fp,"%i %f\n",i,vector->data[i]);

  LALFclose(fp);

  return;
}

