/*
 * $Id$
 *
 * Copyright (C) 2007  Kipp Cannon
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */


#include <string.h>
#include <lal/Date.h>
#include <lal/LALComplex.h>
#include <lal/LALDatatypes.h>
#include <lal/LALStdlib.h>
#include <lal/LALStatusMacros.h>
#include <lal/LALError.h>
#include <lal/Sequence.h>
#include <lal/XLALError.h>


#include <lal/LALRCSID.h>
NRCSID(SEQUENCEC, "$Id$");


/*
 * Shift the bytes in the buffer buff, whose length is length, count bytes to
 * higher addresses.  If the magnitude of count is greater than or equal to
 * that of length, then nothing is done.
 */


static void memshift(void *buff, size_t length, int count)
{
	if(count >= 0) {
		if(length > (size_t) count)
			memmove((char *) buff + count, buff, length - count);
	} else {
		if(length > (size_t) -count)
			memmove(buff, (char *) buff - count, length + count);
	}
}


/*
 * Begin library functions...
 */






void XLALDestroyREAL4Sequence (
	REAL4Sequence *sequence
)
{
	if(sequence)
		XLALFree(sequence->data);
	XLALFree(sequence);
}


REAL4Sequence *XLALCreateREAL4Sequence (
	size_t length
)
{
	static const char func[] = "XLALCreateREAL4Sequence";
	REAL4Sequence *new;
	REAL4 *data;

	new = XLALMalloc(sizeof(*new));
	data = XLALMalloc(length * sizeof(*data));
	/* data == NULL is OK if length == 0 */
	if(!new || (length && !data)) {
		XLALFree(new);
		XLALFree(data);
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	}

	new->data = data;
	new->length = length;

	return new;
}


REAL4Sequence *XLALCutREAL4Sequence (
	REAL4Sequence *sequence,
	size_t first,
	size_t length
)
{
	static const char func[] = "XLALCutREAL4Sequence";
	REAL4Sequence *new = XLALCreateREAL4Sequence (length);
	if(!new)
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	memcpy(new->data, sequence->data + first, length * sizeof(*new->data));

	return new;
}


REAL4Sequence *XLALCopyREAL4Sequence (
	REAL4Sequence *sequence
)
{
	return XLALCutREAL4Sequence (sequence, 0, sequence->length);
}


void XLALShiftREAL4Sequence (
	REAL4Sequence *sequence,
	int count
)
{
	if(!count)
		return;

	memshift(sequence->data, sequence->length * sizeof(*sequence->data), count * (int) sizeof(*sequence->data));
	if((size_t) labs(count) >= sequence->length)
		memset(sequence->data, 0, sequence->length * sizeof(*sequence->data));
	else if(count > 0)
		memset(sequence->data, 0, count * sizeof(*sequence->data));
	else
		memset(sequence->data + sequence->length + count, 0, -count * sizeof(*sequence->data));
}


/* FIXME: this function does not take care to move and zero the least
 * number of bytes possible.  A performance gain would be realized by being
 * more careful. */
REAL4Sequence *XLALResizeREAL4Sequence (
	REAL4Sequence *sequence,
	int first,
	size_t length
)
{
	static const char func[] = "XLALResizeREAL4Sequence";
	REAL4 *new_data;

	if(length > sequence->length) {
		/* need to increase memory */
		new_data = XLALRealloc(sequence->data, length * sizeof(*sequence->data));

		if(new_data) {
			sequence->data = new_data;
			memset(sequence->data + sequence->length, 0, (length - sequence->length) * sizeof(*sequence->data));
			sequence->length = length;
			XLALShiftREAL4Sequence (sequence, -first);
		} else
			XLAL_ERROR_NULL(func, XLAL_EFUNC);
	} else {
		/* do not need to increase memory */
		XLALShiftREAL4Sequence (sequence, -first);
		new_data = XLALRealloc(sequence->data, length * sizeof(*sequence->data));
		if(new_data) {
			sequence->data = new_data;
			sequence->length = length;
		} else
			XLAL_ERROR_NULL(func, XLAL_EFUNC);
	}

	return sequence;
}


REAL4Sequence *XLALShrinkREAL4Sequence (
	REAL4Sequence *sequence,
	size_t first,
	size_t length
)
{
	return XLALResizeREAL4Sequence (sequence, first, length);
}


#if 1
/* FIXME: too bad we aren't C99, or we could include versions for complex
 * data types */
REAL4 XLALREAL4Sum (
	const REAL4 *data,
	size_t first,
	size_t count
)
{
	REAL4 sum = 0;

	for(data += first; count-- > 0; data++)
		sum += *data;

	return sum;
}
#endif


REAL4 XLALREAL4SumSquares (
	const REAL4 *data,
	size_t first,
	size_t count
)
{
	REAL4 sum = 0;

	for(data += first; count-- > 0; data++)
		sum += *data * *data;

	return sum;
}


#if 1
/* FIXME: too bad we aren't C99, or we could include versions for complex
 * data types */
REAL4 XLALREAL4SequenceSum (
	const REAL4Sequence *sequence,
	size_t first,
	size_t count
)
{
	if(first >= sequence->length)
		return 0;
	if(first + count > sequence->length)
		count = sequence->length - first;
	return XLALREAL4Sum (sequence->data, first, count);
}
#endif


REAL4 XLALREAL4SequenceSumSquares (
	const REAL4Sequence *sequence,
	size_t first,
	size_t count
)
{
	if(first >= sequence->length)
		return 0;
	if(first + count > sequence->length)
		count = sequence->length - first;
	return XLALREAL4SumSquares (sequence->data, first, count);
}









void XLALDestroyREAL8Sequence (
	REAL8Sequence *sequence
)
{
	if(sequence)
		XLALFree(sequence->data);
	XLALFree(sequence);
}


REAL8Sequence *XLALCreateREAL8Sequence (
	size_t length
)
{
	static const char func[] = "XLALCreateREAL8Sequence";
	REAL8Sequence *new;
	REAL8 *data;

	new = XLALMalloc(sizeof(*new));
	data = XLALMalloc(length * sizeof(*data));
	/* data == NULL is OK if length == 0 */
	if(!new || (length && !data)) {
		XLALFree(new);
		XLALFree(data);
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	}

	new->data = data;
	new->length = length;

	return new;
}


REAL8Sequence *XLALCutREAL8Sequence (
	REAL8Sequence *sequence,
	size_t first,
	size_t length
)
{
	static const char func[] = "XLALCutREAL8Sequence";
	REAL8Sequence *new = XLALCreateREAL8Sequence (length);
	if(!new)
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	memcpy(new->data, sequence->data + first, length * sizeof(*new->data));

	return new;
}


REAL8Sequence *XLALCopyREAL8Sequence (
	REAL8Sequence *sequence
)
{
	return XLALCutREAL8Sequence (sequence, 0, sequence->length);
}


void XLALShiftREAL8Sequence (
	REAL8Sequence *sequence,
	int count
)
{
	if(!count)
		return;

	memshift(sequence->data, sequence->length * sizeof(*sequence->data), count * (int) sizeof(*sequence->data));
	if((size_t) labs(count) >= sequence->length)
		memset(sequence->data, 0, sequence->length * sizeof(*sequence->data));
	else if(count > 0)
		memset(sequence->data, 0, count * sizeof(*sequence->data));
	else
		memset(sequence->data + sequence->length + count, 0, -count * sizeof(*sequence->data));
}


/* FIXME: this function does not take care to move and zero the least
 * number of bytes possible.  A performance gain would be realized by being
 * more careful. */
REAL8Sequence *XLALResizeREAL8Sequence (
	REAL8Sequence *sequence,
	int first,
	size_t length
)
{
	static const char func[] = "XLALResizeREAL8Sequence";
	REAL8 *new_data;

	if(length > sequence->length) {
		/* need to increase memory */
		new_data = XLALRealloc(sequence->data, length * sizeof(*sequence->data));

		if(new_data) {
			sequence->data = new_data;
			memset(sequence->data + sequence->length, 0, (length - sequence->length) * sizeof(*sequence->data));
			sequence->length = length;
			XLALShiftREAL8Sequence (sequence, -first);
		} else
			XLAL_ERROR_NULL(func, XLAL_EFUNC);
	} else {
		/* do not need to increase memory */
		XLALShiftREAL8Sequence (sequence, -first);
		new_data = XLALRealloc(sequence->data, length * sizeof(*sequence->data));
		if(new_data) {
			sequence->data = new_data;
			sequence->length = length;
		} else
			XLAL_ERROR_NULL(func, XLAL_EFUNC);
	}

	return sequence;
}


REAL8Sequence *XLALShrinkREAL8Sequence (
	REAL8Sequence *sequence,
	size_t first,
	size_t length
)
{
	return XLALResizeREAL8Sequence (sequence, first, length);
}


#if 1
/* FIXME: too bad we aren't C99, or we could include versions for complex
 * data types */
REAL8 XLALREAL8Sum (
	const REAL8 *data,
	size_t first,
	size_t count
)
{
	REAL8 sum = 0;

	for(data += first; count-- > 0; data++)
		sum += *data;

	return sum;
}
#endif


REAL8 XLALREAL8SumSquares (
	const REAL8 *data,
	size_t first,
	size_t count
)
{
	REAL8 sum = 0;

	for(data += first; count-- > 0; data++)
		sum += *data * *data;

	return sum;
}


#if 1
/* FIXME: too bad we aren't C99, or we could include versions for complex
 * data types */
REAL8 XLALREAL8SequenceSum (
	const REAL8Sequence *sequence,
	size_t first,
	size_t count
)
{
	if(first >= sequence->length)
		return 0;
	if(first + count > sequence->length)
		count = sequence->length - first;
	return XLALREAL8Sum (sequence->data, first, count);
}
#endif


REAL8 XLALREAL8SequenceSumSquares (
	const REAL8Sequence *sequence,
	size_t first,
	size_t count
)
{
	if(first >= sequence->length)
		return 0;
	if(first + count > sequence->length)
		count = sequence->length - first;
	return XLALREAL8SumSquares (sequence->data, first, count);
}









void XLALDestroyCOMPLEX8Sequence (
	COMPLEX8Sequence *sequence
)
{
	if(sequence)
		XLALFree(sequence->data);
	XLALFree(sequence);
}


COMPLEX8Sequence *XLALCreateCOMPLEX8Sequence (
	size_t length
)
{
	static const char func[] = "XLALCreateCOMPLEX8Sequence";
	COMPLEX8Sequence *new;
	COMPLEX8 *data;

	new = XLALMalloc(sizeof(*new));
	data = XLALMalloc(length * sizeof(*data));
	/* data == NULL is OK if length == 0 */
	if(!new || (length && !data)) {
		XLALFree(new);
		XLALFree(data);
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	}

	new->data = data;
	new->length = length;

	return new;
}


COMPLEX8Sequence *XLALCutCOMPLEX8Sequence (
	COMPLEX8Sequence *sequence,
	size_t first,
	size_t length
)
{
	static const char func[] = "XLALCutCOMPLEX8Sequence";
	COMPLEX8Sequence *new = XLALCreateCOMPLEX8Sequence (length);
	if(!new)
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	memcpy(new->data, sequence->data + first, length * sizeof(*new->data));

	return new;
}


COMPLEX8Sequence *XLALCopyCOMPLEX8Sequence (
	COMPLEX8Sequence *sequence
)
{
	return XLALCutCOMPLEX8Sequence (sequence, 0, sequence->length);
}


void XLALShiftCOMPLEX8Sequence (
	COMPLEX8Sequence *sequence,
	int count
)
{
	if(!count)
		return;

	memshift(sequence->data, sequence->length * sizeof(*sequence->data), count * (int) sizeof(*sequence->data));
	if((size_t) labs(count) >= sequence->length)
		memset(sequence->data, 0, sequence->length * sizeof(*sequence->data));
	else if(count > 0)
		memset(sequence->data, 0, count * sizeof(*sequence->data));
	else
		memset(sequence->data + sequence->length + count, 0, -count * sizeof(*sequence->data));
}


/* FIXME: this function does not take care to move and zero the least
 * number of bytes possible.  A performance gain would be realized by being
 * more careful. */
COMPLEX8Sequence *XLALResizeCOMPLEX8Sequence (
	COMPLEX8Sequence *sequence,
	int first,
	size_t length
)
{
	static const char func[] = "XLALResizeCOMPLEX8Sequence";
	COMPLEX8 *new_data;

	if(length > sequence->length) {
		/* need to increase memory */
		new_data = XLALRealloc(sequence->data, length * sizeof(*sequence->data));

		if(new_data) {
			sequence->data = new_data;
			memset(sequence->data + sequence->length, 0, (length - sequence->length) * sizeof(*sequence->data));
			sequence->length = length;
			XLALShiftCOMPLEX8Sequence (sequence, -first);
		} else
			XLAL_ERROR_NULL(func, XLAL_EFUNC);
	} else {
		/* do not need to increase memory */
		XLALShiftCOMPLEX8Sequence (sequence, -first);
		new_data = XLALRealloc(sequence->data, length * sizeof(*sequence->data));
		if(new_data) {
			sequence->data = new_data;
			sequence->length = length;
		} else
			XLAL_ERROR_NULL(func, XLAL_EFUNC);
	}

	return sequence;
}


COMPLEX8Sequence *XLALShrinkCOMPLEX8Sequence (
	COMPLEX8Sequence *sequence,
	size_t first,
	size_t length
)
{
	return XLALResizeCOMPLEX8Sequence (sequence, first, length);
}


#if 0
/* FIXME: too bad we aren't C99, or we could include versions for complex
 * data types */
COMPLEX8 XLALCOMPLEX8Sum (
	const COMPLEX8 *data,
	size_t first,
	size_t count
)
{
	COMPLEX8 sum = LAL_COMPLEX8_ZERO;
	

	for(data += first; count-- > 0; data++)
		sum = XLALCOMPLEX8Add(sum, *data);
		

	return sum;
}
#endif


REAL4 XLALCOMPLEX8SumSquares (
	const COMPLEX8 *data,
	size_t first,
	size_t count
)
{
	REAL4 sum = 0;

	for(data += first; count-- > 0; data++)
		sum += XLALCOMPLEX8Abs2(*data);
		

	return sum;
}


#if 0
/* FIXME: too bad we aren't C99, or we could include versions for complex
 * data types */
COMPLEX8 XLALCOMPLEX8SequenceSum (
	const COMPLEX8Sequence *sequence,
	size_t first,
	size_t count
)
{
	if(first >= sequence->length)
		return 0;
	if(first + count > sequence->length)
		count = sequence->length - first;
	return XLALCOMPLEX8Sum (sequence->data, first, count);
}
#endif


REAL4 XLALCOMPLEX8SequenceSumSquares (
	const COMPLEX8Sequence *sequence,
	size_t first,
	size_t count
)
{
	if(first >= sequence->length)
		return 0;
	if(first + count > sequence->length)
		count = sequence->length - first;
	return XLALCOMPLEX8SumSquares (sequence->data, first, count);
}


COMPLEX8Sequence *XLALConjugateCOMPLEX8Sequence (
	COMPLEX8Sequence *sequence
)
{
	unsigned int i;

	for(i = 0; i < sequence->length; i++)
		sequence->data[i].im = -sequence->data[i].im;

	return sequence;
}







void XLALDestroyCOMPLEX16Sequence (
	COMPLEX16Sequence *sequence
)
{
	if(sequence)
		XLALFree(sequence->data);
	XLALFree(sequence);
}


COMPLEX16Sequence *XLALCreateCOMPLEX16Sequence (
	size_t length
)
{
	static const char func[] = "XLALCreateCOMPLEX16Sequence";
	COMPLEX16Sequence *new;
	COMPLEX16 *data;

	new = XLALMalloc(sizeof(*new));
	data = XLALMalloc(length * sizeof(*data));
	/* data == NULL is OK if length == 0 */
	if(!new || (length && !data)) {
		XLALFree(new);
		XLALFree(data);
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	}

	new->data = data;
	new->length = length;

	return new;
}


COMPLEX16Sequence *XLALCutCOMPLEX16Sequence (
	COMPLEX16Sequence *sequence,
	size_t first,
	size_t length
)
{
	static const char func[] = "XLALCutCOMPLEX16Sequence";
	COMPLEX16Sequence *new = XLALCreateCOMPLEX16Sequence (length);
	if(!new)
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	memcpy(new->data, sequence->data + first, length * sizeof(*new->data));

	return new;
}


COMPLEX16Sequence *XLALCopyCOMPLEX16Sequence (
	COMPLEX16Sequence *sequence
)
{
	return XLALCutCOMPLEX16Sequence (sequence, 0, sequence->length);
}


void XLALShiftCOMPLEX16Sequence (
	COMPLEX16Sequence *sequence,
	int count
)
{
	if(!count)
		return;

	memshift(sequence->data, sequence->length * sizeof(*sequence->data), count * (int) sizeof(*sequence->data));
	if((size_t) labs(count) >= sequence->length)
		memset(sequence->data, 0, sequence->length * sizeof(*sequence->data));
	else if(count > 0)
		memset(sequence->data, 0, count * sizeof(*sequence->data));
	else
		memset(sequence->data + sequence->length + count, 0, -count * sizeof(*sequence->data));
}


/* FIXME: this function does not take care to move and zero the least
 * number of bytes possible.  A performance gain would be realized by being
 * more careful. */
COMPLEX16Sequence *XLALResizeCOMPLEX16Sequence (
	COMPLEX16Sequence *sequence,
	int first,
	size_t length
)
{
	static const char func[] = "XLALResizeCOMPLEX16Sequence";
	COMPLEX16 *new_data;

	if(length > sequence->length) {
		/* need to increase memory */
		new_data = XLALRealloc(sequence->data, length * sizeof(*sequence->data));

		if(new_data) {
			sequence->data = new_data;
			memset(sequence->data + sequence->length, 0, (length - sequence->length) * sizeof(*sequence->data));
			sequence->length = length;
			XLALShiftCOMPLEX16Sequence (sequence, -first);
		} else
			XLAL_ERROR_NULL(func, XLAL_EFUNC);
	} else {
		/* do not need to increase memory */
		XLALShiftCOMPLEX16Sequence (sequence, -first);
		new_data = XLALRealloc(sequence->data, length * sizeof(*sequence->data));
		if(new_data) {
			sequence->data = new_data;
			sequence->length = length;
		} else
			XLAL_ERROR_NULL(func, XLAL_EFUNC);
	}

	return sequence;
}


COMPLEX16Sequence *XLALShrinkCOMPLEX16Sequence (
	COMPLEX16Sequence *sequence,
	size_t first,
	size_t length
)
{
	return XLALResizeCOMPLEX16Sequence (sequence, first, length);
}


#if 0
/* FIXME: too bad we aren't C99, or we could include versions for complex
 * data types */
COMPLEX16 XLALCOMPLEX16Sum (
	const COMPLEX16 *data,
	size_t first,
	size_t count
)
{
	COMPLEX16 sum = LAL_COMPLEX16_ZERO;
	

	for(data += first; count-- > 0; data++)
		sum = XLALCOMPLEX16Add(sum, *data);
		

	return sum;
}
#endif


REAL8 XLALCOMPLEX16SumSquares (
	const COMPLEX16 *data,
	size_t first,
	size_t count
)
{
	REAL8 sum = 0;

	for(data += first; count-- > 0; data++)
		sum += XLALCOMPLEX16Abs2(*data);
		

	return sum;
}


#if 0
/* FIXME: too bad we aren't C99, or we could include versions for complex
 * data types */
COMPLEX16 XLALCOMPLEX16SequenceSum (
	const COMPLEX16Sequence *sequence,
	size_t first,
	size_t count
)
{
	if(first >= sequence->length)
		return 0;
	if(first + count > sequence->length)
		count = sequence->length - first;
	return XLALCOMPLEX16Sum (sequence->data, first, count);
}
#endif


REAL8 XLALCOMPLEX16SequenceSumSquares (
	const COMPLEX16Sequence *sequence,
	size_t first,
	size_t count
)
{
	if(first >= sequence->length)
		return 0;
	if(first + count > sequence->length)
		count = sequence->length - first;
	return XLALCOMPLEX16SumSquares (sequence->data, first, count);
}


COMPLEX16Sequence *XLALConjugateCOMPLEX16Sequence (
	COMPLEX16Sequence *sequence
)
{
	unsigned int i;

	for(i = 0; i < sequence->length; i++)
		sequence->data[i].im = -sequence->data[i].im;

	return sequence;
}







void XLALDestroyINT2Sequence (
	INT2Sequence *sequence
)
{
	if(sequence)
		XLALFree(sequence->data);
	XLALFree(sequence);
}


INT2Sequence *XLALCreateINT2Sequence (
	size_t length
)
{
	static const char func[] = "XLALCreateINT2Sequence";
	INT2Sequence *new;
	INT2 *data;

	new = XLALMalloc(sizeof(*new));
	data = XLALMalloc(length * sizeof(*data));
	/* data == NULL is OK if length == 0 */
	if(!new || (length && !data)) {
		XLALFree(new);
		XLALFree(data);
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	}

	new->data = data;
	new->length = length;

	return new;
}


INT2Sequence *XLALCutINT2Sequence (
	INT2Sequence *sequence,
	size_t first,
	size_t length
)
{
	static const char func[] = "XLALCutINT2Sequence";
	INT2Sequence *new = XLALCreateINT2Sequence (length);
	if(!new)
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	memcpy(new->data, sequence->data + first, length * sizeof(*new->data));

	return new;
}


INT2Sequence *XLALCopyINT2Sequence (
	INT2Sequence *sequence
)
{
	return XLALCutINT2Sequence (sequence, 0, sequence->length);
}


void XLALShiftINT2Sequence (
	INT2Sequence *sequence,
	int count
)
{
	if(!count)
		return;

	memshift(sequence->data, sequence->length * sizeof(*sequence->data), count * (int) sizeof(*sequence->data));
	if((size_t) labs(count) >= sequence->length)
		memset(sequence->data, 0, sequence->length * sizeof(*sequence->data));
	else if(count > 0)
		memset(sequence->data, 0, count * sizeof(*sequence->data));
	else
		memset(sequence->data + sequence->length + count, 0, -count * sizeof(*sequence->data));
}


/* FIXME: this function does not take care to move and zero the least
 * number of bytes possible.  A performance gain would be realized by being
 * more careful. */
INT2Sequence *XLALResizeINT2Sequence (
	INT2Sequence *sequence,
	int first,
	size_t length
)
{
	static const char func[] = "XLALResizeINT2Sequence";
	INT2 *new_data;

	if(length > sequence->length) {
		/* need to increase memory */
		new_data = XLALRealloc(sequence->data, length * sizeof(*sequence->data));

		if(new_data) {
			sequence->data = new_data;
			memset(sequence->data + sequence->length, 0, (length - sequence->length) * sizeof(*sequence->data));
			sequence->length = length;
			XLALShiftINT2Sequence (sequence, -first);
		} else
			XLAL_ERROR_NULL(func, XLAL_EFUNC);
	} else {
		/* do not need to increase memory */
		XLALShiftINT2Sequence (sequence, -first);
		new_data = XLALRealloc(sequence->data, length * sizeof(*sequence->data));
		if(new_data) {
			sequence->data = new_data;
			sequence->length = length;
		} else
			XLAL_ERROR_NULL(func, XLAL_EFUNC);
	}

	return sequence;
}


INT2Sequence *XLALShrinkINT2Sequence (
	INT2Sequence *sequence,
	size_t first,
	size_t length
)
{
	return XLALResizeINT2Sequence (sequence, first, length);
}


#if 1
/* FIXME: too bad we aren't C99, or we could include versions for complex
 * data types */
INT2 XLALINT2Sum (
	const INT2 *data,
	size_t first,
	size_t count
)
{
	INT2 sum = 0;

	for(data += first; count-- > 0; data++)
		sum += *data;

	return sum;
}
#endif


UINT2 XLALINT2SumSquares (
	const INT2 *data,
	size_t first,
	size_t count
)
{
	UINT2 sum = 0;

	for(data += first; count-- > 0; data++)
		sum += *data * *data;

	return sum;
}


#if 1
/* FIXME: too bad we aren't C99, or we could include versions for complex
 * data types */
INT2 XLALINT2SequenceSum (
	const INT2Sequence *sequence,
	size_t first,
	size_t count
)
{
	if(first >= sequence->length)
		return 0;
	if(first + count > sequence->length)
		count = sequence->length - first;
	return XLALINT2Sum (sequence->data, first, count);
}
#endif


UINT2 XLALINT2SequenceSumSquares (
	const INT2Sequence *sequence,
	size_t first,
	size_t count
)
{
	if(first >= sequence->length)
		return 0;
	if(first + count > sequence->length)
		count = sequence->length - first;
	return XLALINT2SumSquares (sequence->data, first, count);
}









void XLALDestroyUINT2Sequence (
	UINT2Sequence *sequence
)
{
	if(sequence)
		XLALFree(sequence->data);
	XLALFree(sequence);
}


UINT2Sequence *XLALCreateUINT2Sequence (
	size_t length
)
{
	static const char func[] = "XLALCreateUINT2Sequence";
	UINT2Sequence *new;
	UINT2 *data;

	new = XLALMalloc(sizeof(*new));
	data = XLALMalloc(length * sizeof(*data));
	/* data == NULL is OK if length == 0 */
	if(!new || (length && !data)) {
		XLALFree(new);
		XLALFree(data);
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	}

	new->data = data;
	new->length = length;

	return new;
}


UINT2Sequence *XLALCutUINT2Sequence (
	UINT2Sequence *sequence,
	size_t first,
	size_t length
)
{
	static const char func[] = "XLALCutUINT2Sequence";
	UINT2Sequence *new = XLALCreateUINT2Sequence (length);
	if(!new)
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	memcpy(new->data, sequence->data + first, length * sizeof(*new->data));

	return new;
}


UINT2Sequence *XLALCopyUINT2Sequence (
	UINT2Sequence *sequence
)
{
	return XLALCutUINT2Sequence (sequence, 0, sequence->length);
}


void XLALShiftUINT2Sequence (
	UINT2Sequence *sequence,
	int count
)
{
	if(!count)
		return;

	memshift(sequence->data, sequence->length * sizeof(*sequence->data), count * (int) sizeof(*sequence->data));
	if((size_t) labs(count) >= sequence->length)
		memset(sequence->data, 0, sequence->length * sizeof(*sequence->data));
	else if(count > 0)
		memset(sequence->data, 0, count * sizeof(*sequence->data));
	else
		memset(sequence->data + sequence->length + count, 0, -count * sizeof(*sequence->data));
}


/* FIXME: this function does not take care to move and zero the least
 * number of bytes possible.  A performance gain would be realized by being
 * more careful. */
UINT2Sequence *XLALResizeUINT2Sequence (
	UINT2Sequence *sequence,
	int first,
	size_t length
)
{
	static const char func[] = "XLALResizeUINT2Sequence";
	UINT2 *new_data;

	if(length > sequence->length) {
		/* need to increase memory */
		new_data = XLALRealloc(sequence->data, length * sizeof(*sequence->data));

		if(new_data) {
			sequence->data = new_data;
			memset(sequence->data + sequence->length, 0, (length - sequence->length) * sizeof(*sequence->data));
			sequence->length = length;
			XLALShiftUINT2Sequence (sequence, -first);
		} else
			XLAL_ERROR_NULL(func, XLAL_EFUNC);
	} else {
		/* do not need to increase memory */
		XLALShiftUINT2Sequence (sequence, -first);
		new_data = XLALRealloc(sequence->data, length * sizeof(*sequence->data));
		if(new_data) {
			sequence->data = new_data;
			sequence->length = length;
		} else
			XLAL_ERROR_NULL(func, XLAL_EFUNC);
	}

	return sequence;
}


UINT2Sequence *XLALShrinkUINT2Sequence (
	UINT2Sequence *sequence,
	size_t first,
	size_t length
)
{
	return XLALResizeUINT2Sequence (sequence, first, length);
}


#if 1
/* FIXME: too bad we aren't C99, or we could include versions for complex
 * data types */
UINT2 XLALUINT2Sum (
	const UINT2 *data,
	size_t first,
	size_t count
)
{
	UINT2 sum = 0;

	for(data += first; count-- > 0; data++)
		sum += *data;

	return sum;
}
#endif


UINT2 XLALUINT2SumSquares (
	const UINT2 *data,
	size_t first,
	size_t count
)
{
	UINT2 sum = 0;

	for(data += first; count-- > 0; data++)
		sum += *data * *data;

	return sum;
}


#if 1
/* FIXME: too bad we aren't C99, or we could include versions for complex
 * data types */
UINT2 XLALUINT2SequenceSum (
	const UINT2Sequence *sequence,
	size_t first,
	size_t count
)
{
	if(first >= sequence->length)
		return 0;
	if(first + count > sequence->length)
		count = sequence->length - first;
	return XLALUINT2Sum (sequence->data, first, count);
}
#endif


UINT2 XLALUINT2SequenceSumSquares (
	const UINT2Sequence *sequence,
	size_t first,
	size_t count
)
{
	if(first >= sequence->length)
		return 0;
	if(first + count > sequence->length)
		count = sequence->length - first;
	return XLALUINT2SumSquares (sequence->data, first, count);
}









void XLALDestroyINT4Sequence (
	INT4Sequence *sequence
)
{
	if(sequence)
		XLALFree(sequence->data);
	XLALFree(sequence);
}


INT4Sequence *XLALCreateINT4Sequence (
	size_t length
)
{
	static const char func[] = "XLALCreateINT4Sequence";
	INT4Sequence *new;
	INT4 *data;

	new = XLALMalloc(sizeof(*new));
	data = XLALMalloc(length * sizeof(*data));
	/* data == NULL is OK if length == 0 */
	if(!new || (length && !data)) {
		XLALFree(new);
		XLALFree(data);
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	}

	new->data = data;
	new->length = length;

	return new;
}


INT4Sequence *XLALCutINT4Sequence (
	INT4Sequence *sequence,
	size_t first,
	size_t length
)
{
	static const char func[] = "XLALCutINT4Sequence";
	INT4Sequence *new = XLALCreateINT4Sequence (length);
	if(!new)
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	memcpy(new->data, sequence->data + first, length * sizeof(*new->data));

	return new;
}


INT4Sequence *XLALCopyINT4Sequence (
	INT4Sequence *sequence
)
{
	return XLALCutINT4Sequence (sequence, 0, sequence->length);
}


void XLALShiftINT4Sequence (
	INT4Sequence *sequence,
	int count
)
{
	if(!count)
		return;

	memshift(sequence->data, sequence->length * sizeof(*sequence->data), count * (int) sizeof(*sequence->data));
	if((size_t) labs(count) >= sequence->length)
		memset(sequence->data, 0, sequence->length * sizeof(*sequence->data));
	else if(count > 0)
		memset(sequence->data, 0, count * sizeof(*sequence->data));
	else
		memset(sequence->data + sequence->length + count, 0, -count * sizeof(*sequence->data));
}


/* FIXME: this function does not take care to move and zero the least
 * number of bytes possible.  A performance gain would be realized by being
 * more careful. */
INT4Sequence *XLALResizeINT4Sequence (
	INT4Sequence *sequence,
	int first,
	size_t length
)
{
	static const char func[] = "XLALResizeINT4Sequence";
	INT4 *new_data;

	if(length > sequence->length) {
		/* need to increase memory */
		new_data = XLALRealloc(sequence->data, length * sizeof(*sequence->data));

		if(new_data) {
			sequence->data = new_data;
			memset(sequence->data + sequence->length, 0, (length - sequence->length) * sizeof(*sequence->data));
			sequence->length = length;
			XLALShiftINT4Sequence (sequence, -first);
		} else
			XLAL_ERROR_NULL(func, XLAL_EFUNC);
	} else {
		/* do not need to increase memory */
		XLALShiftINT4Sequence (sequence, -first);
		new_data = XLALRealloc(sequence->data, length * sizeof(*sequence->data));
		if(new_data) {
			sequence->data = new_data;
			sequence->length = length;
		} else
			XLAL_ERROR_NULL(func, XLAL_EFUNC);
	}

	return sequence;
}


INT4Sequence *XLALShrinkINT4Sequence (
	INT4Sequence *sequence,
	size_t first,
	size_t length
)
{
	return XLALResizeINT4Sequence (sequence, first, length);
}


#if 1
/* FIXME: too bad we aren't C99, or we could include versions for complex
 * data types */
INT4 XLALINT4Sum (
	const INT4 *data,
	size_t first,
	size_t count
)
{
	INT4 sum = 0;

	for(data += first; count-- > 0; data++)
		sum += *data;

	return sum;
}
#endif


UINT4 XLALINT4SumSquares (
	const INT4 *data,
	size_t first,
	size_t count
)
{
	UINT4 sum = 0;

	for(data += first; count-- > 0; data++)
		sum += *data * *data;

	return sum;
}


#if 1
/* FIXME: too bad we aren't C99, or we could include versions for complex
 * data types */
INT4 XLALINT4SequenceSum (
	const INT4Sequence *sequence,
	size_t first,
	size_t count
)
{
	if(first >= sequence->length)
		return 0;
	if(first + count > sequence->length)
		count = sequence->length - first;
	return XLALINT4Sum (sequence->data, first, count);
}
#endif


UINT4 XLALINT4SequenceSumSquares (
	const INT4Sequence *sequence,
	size_t first,
	size_t count
)
{
	if(first >= sequence->length)
		return 0;
	if(first + count > sequence->length)
		count = sequence->length - first;
	return XLALINT4SumSquares (sequence->data, first, count);
}









void XLALDestroyUINT4Sequence (
	UINT4Sequence *sequence
)
{
	if(sequence)
		XLALFree(sequence->data);
	XLALFree(sequence);
}


UINT4Sequence *XLALCreateUINT4Sequence (
	size_t length
)
{
	static const char func[] = "XLALCreateUINT4Sequence";
	UINT4Sequence *new;
	UINT4 *data;

	new = XLALMalloc(sizeof(*new));
	data = XLALMalloc(length * sizeof(*data));
	/* data == NULL is OK if length == 0 */
	if(!new || (length && !data)) {
		XLALFree(new);
		XLALFree(data);
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	}

	new->data = data;
	new->length = length;

	return new;
}


UINT4Sequence *XLALCutUINT4Sequence (
	UINT4Sequence *sequence,
	size_t first,
	size_t length
)
{
	static const char func[] = "XLALCutUINT4Sequence";
	UINT4Sequence *new = XLALCreateUINT4Sequence (length);
	if(!new)
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	memcpy(new->data, sequence->data + first, length * sizeof(*new->data));

	return new;
}


UINT4Sequence *XLALCopyUINT4Sequence (
	UINT4Sequence *sequence
)
{
	return XLALCutUINT4Sequence (sequence, 0, sequence->length);
}


void XLALShiftUINT4Sequence (
	UINT4Sequence *sequence,
	int count
)
{
	if(!count)
		return;

	memshift(sequence->data, sequence->length * sizeof(*sequence->data), count * (int) sizeof(*sequence->data));
	if((size_t) labs(count) >= sequence->length)
		memset(sequence->data, 0, sequence->length * sizeof(*sequence->data));
	else if(count > 0)
		memset(sequence->data, 0, count * sizeof(*sequence->data));
	else
		memset(sequence->data + sequence->length + count, 0, -count * sizeof(*sequence->data));
}


/* FIXME: this function does not take care to move and zero the least
 * number of bytes possible.  A performance gain would be realized by being
 * more careful. */
UINT4Sequence *XLALResizeUINT4Sequence (
	UINT4Sequence *sequence,
	int first,
	size_t length
)
{
	static const char func[] = "XLALResizeUINT4Sequence";
	UINT4 *new_data;

	if(length > sequence->length) {
		/* need to increase memory */
		new_data = XLALRealloc(sequence->data, length * sizeof(*sequence->data));

		if(new_data) {
			sequence->data = new_data;
			memset(sequence->data + sequence->length, 0, (length - sequence->length) * sizeof(*sequence->data));
			sequence->length = length;
			XLALShiftUINT4Sequence (sequence, -first);
		} else
			XLAL_ERROR_NULL(func, XLAL_EFUNC);
	} else {
		/* do not need to increase memory */
		XLALShiftUINT4Sequence (sequence, -first);
		new_data = XLALRealloc(sequence->data, length * sizeof(*sequence->data));
		if(new_data) {
			sequence->data = new_data;
			sequence->length = length;
		} else
			XLAL_ERROR_NULL(func, XLAL_EFUNC);
	}

	return sequence;
}


UINT4Sequence *XLALShrinkUINT4Sequence (
	UINT4Sequence *sequence,
	size_t first,
	size_t length
)
{
	return XLALResizeUINT4Sequence (sequence, first, length);
}


#if 1
/* FIXME: too bad we aren't C99, or we could include versions for complex
 * data types */
UINT4 XLALUINT4Sum (
	const UINT4 *data,
	size_t first,
	size_t count
)
{
	UINT4 sum = 0;

	for(data += first; count-- > 0; data++)
		sum += *data;

	return sum;
}
#endif


UINT4 XLALUINT4SumSquares (
	const UINT4 *data,
	size_t first,
	size_t count
)
{
	UINT4 sum = 0;

	for(data += first; count-- > 0; data++)
		sum += *data * *data;

	return sum;
}


#if 1
/* FIXME: too bad we aren't C99, or we could include versions for complex
 * data types */
UINT4 XLALUINT4SequenceSum (
	const UINT4Sequence *sequence,
	size_t first,
	size_t count
)
{
	if(first >= sequence->length)
		return 0;
	if(first + count > sequence->length)
		count = sequence->length - first;
	return XLALUINT4Sum (sequence->data, first, count);
}
#endif


UINT4 XLALUINT4SequenceSumSquares (
	const UINT4Sequence *sequence,
	size_t first,
	size_t count
)
{
	if(first >= sequence->length)
		return 0;
	if(first + count > sequence->length)
		count = sequence->length - first;
	return XLALUINT4SumSquares (sequence->data, first, count);
}









void XLALDestroyINT8Sequence (
	INT8Sequence *sequence
)
{
	if(sequence)
		XLALFree(sequence->data);
	XLALFree(sequence);
}


INT8Sequence *XLALCreateINT8Sequence (
	size_t length
)
{
	static const char func[] = "XLALCreateINT8Sequence";
	INT8Sequence *new;
	INT8 *data;

	new = XLALMalloc(sizeof(*new));
	data = XLALMalloc(length * sizeof(*data));
	/* data == NULL is OK if length == 0 */
	if(!new || (length && !data)) {
		XLALFree(new);
		XLALFree(data);
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	}

	new->data = data;
	new->length = length;

	return new;
}


INT8Sequence *XLALCutINT8Sequence (
	INT8Sequence *sequence,
	size_t first,
	size_t length
)
{
	static const char func[] = "XLALCutINT8Sequence";
	INT8Sequence *new = XLALCreateINT8Sequence (length);
	if(!new)
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	memcpy(new->data, sequence->data + first, length * sizeof(*new->data));

	return new;
}


INT8Sequence *XLALCopyINT8Sequence (
	INT8Sequence *sequence
)
{
	return XLALCutINT8Sequence (sequence, 0, sequence->length);
}


void XLALShiftINT8Sequence (
	INT8Sequence *sequence,
	int count
)
{
	if(!count)
		return;

	memshift(sequence->data, sequence->length * sizeof(*sequence->data), count * (int) sizeof(*sequence->data));
	if((size_t) labs(count) >= sequence->length)
		memset(sequence->data, 0, sequence->length * sizeof(*sequence->data));
	else if(count > 0)
		memset(sequence->data, 0, count * sizeof(*sequence->data));
	else
		memset(sequence->data + sequence->length + count, 0, -count * sizeof(*sequence->data));
}


/* FIXME: this function does not take care to move and zero the least
 * number of bytes possible.  A performance gain would be realized by being
 * more careful. */
INT8Sequence *XLALResizeINT8Sequence (
	INT8Sequence *sequence,
	int first,
	size_t length
)
{
	static const char func[] = "XLALResizeINT8Sequence";
	INT8 *new_data;

	if(length > sequence->length) {
		/* need to increase memory */
		new_data = XLALRealloc(sequence->data, length * sizeof(*sequence->data));

		if(new_data) {
			sequence->data = new_data;
			memset(sequence->data + sequence->length, 0, (length - sequence->length) * sizeof(*sequence->data));
			sequence->length = length;
			XLALShiftINT8Sequence (sequence, -first);
		} else
			XLAL_ERROR_NULL(func, XLAL_EFUNC);
	} else {
		/* do not need to increase memory */
		XLALShiftINT8Sequence (sequence, -first);
		new_data = XLALRealloc(sequence->data, length * sizeof(*sequence->data));
		if(new_data) {
			sequence->data = new_data;
			sequence->length = length;
		} else
			XLAL_ERROR_NULL(func, XLAL_EFUNC);
	}

	return sequence;
}


INT8Sequence *XLALShrinkINT8Sequence (
	INT8Sequence *sequence,
	size_t first,
	size_t length
)
{
	return XLALResizeINT8Sequence (sequence, first, length);
}


#if 1
/* FIXME: too bad we aren't C99, or we could include versions for complex
 * data types */
INT8 XLALINT8Sum (
	const INT8 *data,
	size_t first,
	size_t count
)
{
	INT8 sum = 0;

	for(data += first; count-- > 0; data++)
		sum += *data;

	return sum;
}
#endif


UINT8 XLALINT8SumSquares (
	const INT8 *data,
	size_t first,
	size_t count
)
{
	UINT8 sum = 0;

	for(data += first; count-- > 0; data++)
		sum += *data * *data;

	return sum;
}


#if 1
/* FIXME: too bad we aren't C99, or we could include versions for complex
 * data types */
INT8 XLALINT8SequenceSum (
	const INT8Sequence *sequence,
	size_t first,
	size_t count
)
{
	if(first >= sequence->length)
		return 0;
	if(first + count > sequence->length)
		count = sequence->length - first;
	return XLALINT8Sum (sequence->data, first, count);
}
#endif


UINT8 XLALINT8SequenceSumSquares (
	const INT8Sequence *sequence,
	size_t first,
	size_t count
)
{
	if(first >= sequence->length)
		return 0;
	if(first + count > sequence->length)
		count = sequence->length - first;
	return XLALINT8SumSquares (sequence->data, first, count);
}









void XLALDestroyUINT8Sequence (
	UINT8Sequence *sequence
)
{
	if(sequence)
		XLALFree(sequence->data);
	XLALFree(sequence);
}


UINT8Sequence *XLALCreateUINT8Sequence (
	size_t length
)
{
	static const char func[] = "XLALCreateUINT8Sequence";
	UINT8Sequence *new;
	UINT8 *data;

	new = XLALMalloc(sizeof(*new));
	data = XLALMalloc(length * sizeof(*data));
	/* data == NULL is OK if length == 0 */
	if(!new || (length && !data)) {
		XLALFree(new);
		XLALFree(data);
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	}

	new->data = data;
	new->length = length;

	return new;
}


UINT8Sequence *XLALCutUINT8Sequence (
	UINT8Sequence *sequence,
	size_t first,
	size_t length
)
{
	static const char func[] = "XLALCutUINT8Sequence";
	UINT8Sequence *new = XLALCreateUINT8Sequence (length);
	if(!new)
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	memcpy(new->data, sequence->data + first, length * sizeof(*new->data));

	return new;
}


UINT8Sequence *XLALCopyUINT8Sequence (
	UINT8Sequence *sequence
)
{
	return XLALCutUINT8Sequence (sequence, 0, sequence->length);
}


void XLALShiftUINT8Sequence (
	UINT8Sequence *sequence,
	int count
)
{
	if(!count)
		return;

	memshift(sequence->data, sequence->length * sizeof(*sequence->data), count * (int) sizeof(*sequence->data));
	if((size_t) labs(count) >= sequence->length)
		memset(sequence->data, 0, sequence->length * sizeof(*sequence->data));
	else if(count > 0)
		memset(sequence->data, 0, count * sizeof(*sequence->data));
	else
		memset(sequence->data + sequence->length + count, 0, -count * sizeof(*sequence->data));
}


/* FIXME: this function does not take care to move and zero the least
 * number of bytes possible.  A performance gain would be realized by being
 * more careful. */
UINT8Sequence *XLALResizeUINT8Sequence (
	UINT8Sequence *sequence,
	int first,
	size_t length
)
{
	static const char func[] = "XLALResizeUINT8Sequence";
	UINT8 *new_data;

	if(length > sequence->length) {
		/* need to increase memory */
		new_data = XLALRealloc(sequence->data, length * sizeof(*sequence->data));

		if(new_data) {
			sequence->data = new_data;
			memset(sequence->data + sequence->length, 0, (length - sequence->length) * sizeof(*sequence->data));
			sequence->length = length;
			XLALShiftUINT8Sequence (sequence, -first);
		} else
			XLAL_ERROR_NULL(func, XLAL_EFUNC);
	} else {
		/* do not need to increase memory */
		XLALShiftUINT8Sequence (sequence, -first);
		new_data = XLALRealloc(sequence->data, length * sizeof(*sequence->data));
		if(new_data) {
			sequence->data = new_data;
			sequence->length = length;
		} else
			XLAL_ERROR_NULL(func, XLAL_EFUNC);
	}

	return sequence;
}


UINT8Sequence *XLALShrinkUINT8Sequence (
	UINT8Sequence *sequence,
	size_t first,
	size_t length
)
{
	return XLALResizeUINT8Sequence (sequence, first, length);
}


#if 1
/* FIXME: too bad we aren't C99, or we could include versions for complex
 * data types */
UINT8 XLALUINT8Sum (
	const UINT8 *data,
	size_t first,
	size_t count
)
{
	UINT8 sum = 0;

	for(data += first; count-- > 0; data++)
		sum += *data;

	return sum;
}
#endif


UINT8 XLALUINT8SumSquares (
	const UINT8 *data,
	size_t first,
	size_t count
)
{
	UINT8 sum = 0;

	for(data += first; count-- > 0; data++)
		sum += *data * *data;

	return sum;
}


#if 1
/* FIXME: too bad we aren't C99, or we could include versions for complex
 * data types */
UINT8 XLALUINT8SequenceSum (
	const UINT8Sequence *sequence,
	size_t first,
	size_t count
)
{
	if(first >= sequence->length)
		return 0;
	if(first + count > sequence->length)
		count = sequence->length - first;
	return XLALUINT8Sum (sequence->data, first, count);
}
#endif


UINT8 XLALUINT8SequenceSumSquares (
	const UINT8Sequence *sequence,
	size_t first,
	size_t count
)
{
	if(first >= sequence->length)
		return 0;
	if(first + count > sequence->length)
		count = sequence->length - first;
	return XLALUINT8SumSquares (sequence->data, first, count);
}




