/*
 * $Id$
 *
 * Copyright (C) 2007  Kipp Cannon
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General
 * Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */


#include <math.h>
#include <string.h>
#include <lal/Date.h>
#include <lal/LALDatatypes.h>
#include <lal/LALStdlib.h>
#include <lal/Sequence.h>
#include <lal/TimeSeries.h>
#include <lal/Units.h>
#include <lal/XLALError.h>


#include <lal/LALRCSID.h>
NRCSID(TIMESERIESC, "$Id$");






void XLALDestroyREAL4TimeSeries (
	REAL4TimeSeries *series
)
{
	if(series)
		XLALDestroyREAL4Sequence (series->data);
	XLALFree(series);
}


REAL4TimeSeries *XLALCreateREAL4TimeSeries (
	const CHAR *name,
	const LIGOTimeGPS *epoch,
	REAL8 f0,
	REAL8 deltaT,
	const LALUnit *sampleUnits,
	size_t length
)
{
	static const char func[] = "XLALCreateREAL4TimeSeries";
	REAL4TimeSeries *new;
	REAL4Sequence *sequence;

	new = XLALMalloc(sizeof(*new));
	sequence = XLALCreateREAL4Sequence (length);
	if(!new || !sequence) {
		XLALFree(new);
		XLALDestroyREAL4Sequence (sequence);
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	}

	if(name)
		strncpy(new->name, name, LALNameLength);
	else
		new->name[0] = '\0';
	new->epoch = *epoch;
	new->f0 = f0;
	new->deltaT = deltaT;
	new->sampleUnits = *sampleUnits;
	new->data = sequence;

	return new;
}


REAL4TimeSeries *XLALCutREAL4TimeSeries (
	const REAL4TimeSeries *series,
	size_t first,
	size_t length
)
{
	static const char func[] = "XLALCutREAL4TimeSeries";
	REAL4TimeSeries *new;
	REAL4Sequence *sequence;

	new = XLALMalloc(sizeof(*new));
	sequence = XLALCutREAL4Sequence (series->data, first, length);
	if(!new || !sequence) {
		XLALFree(new);
		XLALDestroyREAL4Sequence (sequence);
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	}

	*new = *series;
	new->data = sequence;
	XLALGPSAdd(&new->epoch, first * new->deltaT);

	return new;
}


REAL4TimeSeries *XLALResizeREAL4TimeSeries (
	REAL4TimeSeries *series,
	int first,
	size_t length
)
{
	static const char func[] = "XLALResizeREAL4TimeSeries";

	if(!XLALResizeREAL4Sequence (series->data, first, length))
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	XLALGPSAdd(&series->epoch, first * series->deltaT);

	return series;
}


REAL4TimeSeries *XLALShrinkREAL4TimeSeries (
	REAL4TimeSeries *series,
	size_t first,
	size_t length
)
{
	static const char func[] = "XLALShrinkREAL4TimeSeries";

	if(!XLALResizeREAL4TimeSeries (series, first, length))
		XLAL_ERROR_NULL(func, XLAL_EFUNC);

	return series;
}


REAL4TimeSeries *XLALAddREAL4TimeSeries (
	REAL4TimeSeries *arg1,
	const REAL4TimeSeries *arg2
)
{
	static const char func[] = "XLALAddREAL4TimeSeries";
	REAL8 Delta_epoch = XLALGPSDiff(&arg2->epoch, &arg1->epoch);
	/* number of arg1 units per arg2 unit.  XLALUnitRatio() returns the
	 * number one obtains when one divides 1 of the first argument by 1
	 * of the second argument, for example if arg2 is in m and arg1 is
	 * in cm then unit_ratio = 100.0 */
	REAL8 unit_ratio = XLALUnitRatio(&arg2->sampleUnits, &arg1->sampleUnits);
	unsigned i, j;

	/* make sure arguments are compatible */
	if(XLALIsREAL8FailNaN(unit_ratio)) {
		XLALPrintError("%s(): incompatible units\n", func);
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	}
	if(arg1->f0 != arg2->f0) {
		XLALPrintError("%s(): incompatible heterodyne frequencies\n", func);
		XLAL_ERROR_NULL(func, XLAL_EDATA);
	}
	if(arg1->deltaT != arg2->deltaT) {
		XLALPrintError("%s(): incompatible sample periods\n", func);
		XLAL_ERROR_NULL(func, XLAL_EDATA);
	}

	/* set start indexes */
	if(Delta_epoch >= 0) {
		i = floor(Delta_epoch / arg1->deltaT + 0.5);
		j = 0;
	} else {
		i = 0;
		j = floor(-Delta_epoch / arg2->deltaT + 0.5);
	}

	/* add arg2 to arg1, adjusting the units */
	for(; i < arg1->data->length && j < arg2->data->length; i++, j++) {
		arg1->data->data[i] += arg2->data->data[j] * unit_ratio;
	}

	return arg1;
}







void XLALDestroyREAL8TimeSeries (
	REAL8TimeSeries *series
)
{
	if(series)
		XLALDestroyREAL8Sequence (series->data);
	XLALFree(series);
}


REAL8TimeSeries *XLALCreateREAL8TimeSeries (
	const CHAR *name,
	const LIGOTimeGPS *epoch,
	REAL8 f0,
	REAL8 deltaT,
	const LALUnit *sampleUnits,
	size_t length
)
{
	static const char func[] = "XLALCreateREAL8TimeSeries";
	REAL8TimeSeries *new;
	REAL8Sequence *sequence;

	new = XLALMalloc(sizeof(*new));
	sequence = XLALCreateREAL8Sequence (length);
	if(!new || !sequence) {
		XLALFree(new);
		XLALDestroyREAL8Sequence (sequence);
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	}

	if(name)
		strncpy(new->name, name, LALNameLength);
	else
		new->name[0] = '\0';
	new->epoch = *epoch;
	new->f0 = f0;
	new->deltaT = deltaT;
	new->sampleUnits = *sampleUnits;
	new->data = sequence;

	return new;
}


REAL8TimeSeries *XLALCutREAL8TimeSeries (
	const REAL8TimeSeries *series,
	size_t first,
	size_t length
)
{
	static const char func[] = "XLALCutREAL8TimeSeries";
	REAL8TimeSeries *new;
	REAL8Sequence *sequence;

	new = XLALMalloc(sizeof(*new));
	sequence = XLALCutREAL8Sequence (series->data, first, length);
	if(!new || !sequence) {
		XLALFree(new);
		XLALDestroyREAL8Sequence (sequence);
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	}

	*new = *series;
	new->data = sequence;
	XLALGPSAdd(&new->epoch, first * new->deltaT);

	return new;
}


REAL8TimeSeries *XLALResizeREAL8TimeSeries (
	REAL8TimeSeries *series,
	int first,
	size_t length
)
{
	static const char func[] = "XLALResizeREAL8TimeSeries";

	if(!XLALResizeREAL8Sequence (series->data, first, length))
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	XLALGPSAdd(&series->epoch, first * series->deltaT);

	return series;
}


REAL8TimeSeries *XLALShrinkREAL8TimeSeries (
	REAL8TimeSeries *series,
	size_t first,
	size_t length
)
{
	static const char func[] = "XLALShrinkREAL8TimeSeries";

	if(!XLALResizeREAL8TimeSeries (series, first, length))
		XLAL_ERROR_NULL(func, XLAL_EFUNC);

	return series;
}


REAL8TimeSeries *XLALAddREAL8TimeSeries (
	REAL8TimeSeries *arg1,
	const REAL8TimeSeries *arg2
)
{
	static const char func[] = "XLALAddREAL8TimeSeries";
	REAL8 Delta_epoch = XLALGPSDiff(&arg2->epoch, &arg1->epoch);
	/* number of arg1 units per arg2 unit.  XLALUnitRatio() returns the
	 * number one obtains when one divides 1 of the first argument by 1
	 * of the second argument, for example if arg2 is in m and arg1 is
	 * in cm then unit_ratio = 100.0 */
	REAL8 unit_ratio = XLALUnitRatio(&arg2->sampleUnits, &arg1->sampleUnits);
	unsigned i, j;

	/* make sure arguments are compatible */
	if(XLALIsREAL8FailNaN(unit_ratio)) {
		XLALPrintError("%s(): incompatible units\n", func);
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	}
	if(arg1->f0 != arg2->f0) {
		XLALPrintError("%s(): incompatible heterodyne frequencies\n", func);
		XLAL_ERROR_NULL(func, XLAL_EDATA);
	}
	if(arg1->deltaT != arg2->deltaT) {
		XLALPrintError("%s(): incompatible sample periods\n", func);
		XLAL_ERROR_NULL(func, XLAL_EDATA);
	}

	/* set start indexes */
	if(Delta_epoch >= 0) {
		i = floor(Delta_epoch / arg1->deltaT + 0.5);
		j = 0;
	} else {
		i = 0;
		j = floor(-Delta_epoch / arg2->deltaT + 0.5);
	}

	/* add arg2 to arg1, adjusting the units */
	for(; i < arg1->data->length && j < arg2->data->length; i++, j++) {
		arg1->data->data[i] += arg2->data->data[j] * unit_ratio;
	}

	return arg1;
}







void XLALDestroyCOMPLEX8TimeSeries (
	COMPLEX8TimeSeries *series
)
{
	if(series)
		XLALDestroyCOMPLEX8Sequence (series->data);
	XLALFree(series);
}


COMPLEX8TimeSeries *XLALCreateCOMPLEX8TimeSeries (
	const CHAR *name,
	const LIGOTimeGPS *epoch,
	REAL8 f0,
	REAL8 deltaT,
	const LALUnit *sampleUnits,
	size_t length
)
{
	static const char func[] = "XLALCreateCOMPLEX8TimeSeries";
	COMPLEX8TimeSeries *new;
	COMPLEX8Sequence *sequence;

	new = XLALMalloc(sizeof(*new));
	sequence = XLALCreateCOMPLEX8Sequence (length);
	if(!new || !sequence) {
		XLALFree(new);
		XLALDestroyCOMPLEX8Sequence (sequence);
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	}

	if(name)
		strncpy(new->name, name, LALNameLength);
	else
		new->name[0] = '\0';
	new->epoch = *epoch;
	new->f0 = f0;
	new->deltaT = deltaT;
	new->sampleUnits = *sampleUnits;
	new->data = sequence;

	return new;
}


COMPLEX8TimeSeries *XLALCutCOMPLEX8TimeSeries (
	const COMPLEX8TimeSeries *series,
	size_t first,
	size_t length
)
{
	static const char func[] = "XLALCutCOMPLEX8TimeSeries";
	COMPLEX8TimeSeries *new;
	COMPLEX8Sequence *sequence;

	new = XLALMalloc(sizeof(*new));
	sequence = XLALCutCOMPLEX8Sequence (series->data, first, length);
	if(!new || !sequence) {
		XLALFree(new);
		XLALDestroyCOMPLEX8Sequence (sequence);
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	}

	*new = *series;
	new->data = sequence;
	XLALGPSAdd(&new->epoch, first * new->deltaT);

	return new;
}


COMPLEX8TimeSeries *XLALResizeCOMPLEX8TimeSeries (
	COMPLEX8TimeSeries *series,
	int first,
	size_t length
)
{
	static const char func[] = "XLALResizeCOMPLEX8TimeSeries";

	if(!XLALResizeCOMPLEX8Sequence (series->data, first, length))
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	XLALGPSAdd(&series->epoch, first * series->deltaT);

	return series;
}


COMPLEX8TimeSeries *XLALShrinkCOMPLEX8TimeSeries (
	COMPLEX8TimeSeries *series,
	size_t first,
	size_t length
)
{
	static const char func[] = "XLALShrinkCOMPLEX8TimeSeries";

	if(!XLALResizeCOMPLEX8TimeSeries (series, first, length))
		XLAL_ERROR_NULL(func, XLAL_EFUNC);

	return series;
}


COMPLEX8TimeSeries *XLALAddCOMPLEX8TimeSeries (
	COMPLEX8TimeSeries *arg1,
	const COMPLEX8TimeSeries *arg2
)
{
	static const char func[] = "XLALAddCOMPLEX8TimeSeries";
	REAL8 Delta_epoch = XLALGPSDiff(&arg2->epoch, &arg1->epoch);
	/* number of arg1 units per arg2 unit.  XLALUnitRatio() returns the
	 * number one obtains when one divides 1 of the first argument by 1
	 * of the second argument, for example if arg2 is in m and arg1 is
	 * in cm then unit_ratio = 100.0 */
	REAL8 unit_ratio = XLALUnitRatio(&arg2->sampleUnits, &arg1->sampleUnits);
	unsigned i, j;

	/* make sure arguments are compatible */
	if(XLALIsREAL8FailNaN(unit_ratio)) {
		XLALPrintError("%s(): incompatible units\n", func);
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	}
	if(arg1->f0 != arg2->f0) {
		XLALPrintError("%s(): incompatible heterodyne frequencies\n", func);
		XLAL_ERROR_NULL(func, XLAL_EDATA);
	}
	if(arg1->deltaT != arg2->deltaT) {
		XLALPrintError("%s(): incompatible sample periods\n", func);
		XLAL_ERROR_NULL(func, XLAL_EDATA);
	}

	/* set start indexes */
	if(Delta_epoch >= 0) {
		i = floor(Delta_epoch / arg1->deltaT + 0.5);
		j = 0;
	} else {
		i = 0;
		j = floor(-Delta_epoch / arg2->deltaT + 0.5);
	}

	/* add arg2 to arg1, adjusting the units */
	for(; i < arg1->data->length && j < arg2->data->length; i++, j++) {
		arg1->data->data[i].re += arg2->data->data[j].re * unit_ratio;
		arg1->data->data[i].im += arg2->data->data[j].im * unit_ratio;
		
	}

	return arg1;
}







void XLALDestroyCOMPLEX16TimeSeries (
	COMPLEX16TimeSeries *series
)
{
	if(series)
		XLALDestroyCOMPLEX16Sequence (series->data);
	XLALFree(series);
}


COMPLEX16TimeSeries *XLALCreateCOMPLEX16TimeSeries (
	const CHAR *name,
	const LIGOTimeGPS *epoch,
	REAL8 f0,
	REAL8 deltaT,
	const LALUnit *sampleUnits,
	size_t length
)
{
	static const char func[] = "XLALCreateCOMPLEX16TimeSeries";
	COMPLEX16TimeSeries *new;
	COMPLEX16Sequence *sequence;

	new = XLALMalloc(sizeof(*new));
	sequence = XLALCreateCOMPLEX16Sequence (length);
	if(!new || !sequence) {
		XLALFree(new);
		XLALDestroyCOMPLEX16Sequence (sequence);
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	}

	if(name)
		strncpy(new->name, name, LALNameLength);
	else
		new->name[0] = '\0';
	new->epoch = *epoch;
	new->f0 = f0;
	new->deltaT = deltaT;
	new->sampleUnits = *sampleUnits;
	new->data = sequence;

	return new;
}


COMPLEX16TimeSeries *XLALCutCOMPLEX16TimeSeries (
	const COMPLEX16TimeSeries *series,
	size_t first,
	size_t length
)
{
	static const char func[] = "XLALCutCOMPLEX16TimeSeries";
	COMPLEX16TimeSeries *new;
	COMPLEX16Sequence *sequence;

	new = XLALMalloc(sizeof(*new));
	sequence = XLALCutCOMPLEX16Sequence (series->data, first, length);
	if(!new || !sequence) {
		XLALFree(new);
		XLALDestroyCOMPLEX16Sequence (sequence);
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	}

	*new = *series;
	new->data = sequence;
	XLALGPSAdd(&new->epoch, first * new->deltaT);

	return new;
}


COMPLEX16TimeSeries *XLALResizeCOMPLEX16TimeSeries (
	COMPLEX16TimeSeries *series,
	int first,
	size_t length
)
{
	static const char func[] = "XLALResizeCOMPLEX16TimeSeries";

	if(!XLALResizeCOMPLEX16Sequence (series->data, first, length))
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	XLALGPSAdd(&series->epoch, first * series->deltaT);

	return series;
}


COMPLEX16TimeSeries *XLALShrinkCOMPLEX16TimeSeries (
	COMPLEX16TimeSeries *series,
	size_t first,
	size_t length
)
{
	static const char func[] = "XLALShrinkCOMPLEX16TimeSeries";

	if(!XLALResizeCOMPLEX16TimeSeries (series, first, length))
		XLAL_ERROR_NULL(func, XLAL_EFUNC);

	return series;
}


COMPLEX16TimeSeries *XLALAddCOMPLEX16TimeSeries (
	COMPLEX16TimeSeries *arg1,
	const COMPLEX16TimeSeries *arg2
)
{
	static const char func[] = "XLALAddCOMPLEX16TimeSeries";
	REAL8 Delta_epoch = XLALGPSDiff(&arg2->epoch, &arg1->epoch);
	/* number of arg1 units per arg2 unit.  XLALUnitRatio() returns the
	 * number one obtains when one divides 1 of the first argument by 1
	 * of the second argument, for example if arg2 is in m and arg1 is
	 * in cm then unit_ratio = 100.0 */
	REAL8 unit_ratio = XLALUnitRatio(&arg2->sampleUnits, &arg1->sampleUnits);
	unsigned i, j;

	/* make sure arguments are compatible */
	if(XLALIsREAL8FailNaN(unit_ratio)) {
		XLALPrintError("%s(): incompatible units\n", func);
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	}
	if(arg1->f0 != arg2->f0) {
		XLALPrintError("%s(): incompatible heterodyne frequencies\n", func);
		XLAL_ERROR_NULL(func, XLAL_EDATA);
	}
	if(arg1->deltaT != arg2->deltaT) {
		XLALPrintError("%s(): incompatible sample periods\n", func);
		XLAL_ERROR_NULL(func, XLAL_EDATA);
	}

	/* set start indexes */
	if(Delta_epoch >= 0) {
		i = floor(Delta_epoch / arg1->deltaT + 0.5);
		j = 0;
	} else {
		i = 0;
		j = floor(-Delta_epoch / arg2->deltaT + 0.5);
	}

	/* add arg2 to arg1, adjusting the units */
	for(; i < arg1->data->length && j < arg2->data->length; i++, j++) {
		arg1->data->data[i].re += arg2->data->data[j].re * unit_ratio;
		arg1->data->data[i].im += arg2->data->data[j].im * unit_ratio;
		
	}

	return arg1;
}







void XLALDestroyINT2TimeSeries (
	INT2TimeSeries *series
)
{
	if(series)
		XLALDestroyINT2Sequence (series->data);
	XLALFree(series);
}


INT2TimeSeries *XLALCreateINT2TimeSeries (
	const CHAR *name,
	const LIGOTimeGPS *epoch,
	REAL8 f0,
	REAL8 deltaT,
	const LALUnit *sampleUnits,
	size_t length
)
{
	static const char func[] = "XLALCreateINT2TimeSeries";
	INT2TimeSeries *new;
	INT2Sequence *sequence;

	new = XLALMalloc(sizeof(*new));
	sequence = XLALCreateINT2Sequence (length);
	if(!new || !sequence) {
		XLALFree(new);
		XLALDestroyINT2Sequence (sequence);
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	}

	if(name)
		strncpy(new->name, name, LALNameLength);
	else
		new->name[0] = '\0';
	new->epoch = *epoch;
	new->f0 = f0;
	new->deltaT = deltaT;
	new->sampleUnits = *sampleUnits;
	new->data = sequence;

	return new;
}


INT2TimeSeries *XLALCutINT2TimeSeries (
	const INT2TimeSeries *series,
	size_t first,
	size_t length
)
{
	static const char func[] = "XLALCutINT2TimeSeries";
	INT2TimeSeries *new;
	INT2Sequence *sequence;

	new = XLALMalloc(sizeof(*new));
	sequence = XLALCutINT2Sequence (series->data, first, length);
	if(!new || !sequence) {
		XLALFree(new);
		XLALDestroyINT2Sequence (sequence);
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	}

	*new = *series;
	new->data = sequence;
	XLALGPSAdd(&new->epoch, first * new->deltaT);

	return new;
}


INT2TimeSeries *XLALResizeINT2TimeSeries (
	INT2TimeSeries *series,
	int first,
	size_t length
)
{
	static const char func[] = "XLALResizeINT2TimeSeries";

	if(!XLALResizeINT2Sequence (series->data, first, length))
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	XLALGPSAdd(&series->epoch, first * series->deltaT);

	return series;
}


INT2TimeSeries *XLALShrinkINT2TimeSeries (
	INT2TimeSeries *series,
	size_t first,
	size_t length
)
{
	static const char func[] = "XLALShrinkINT2TimeSeries";

	if(!XLALResizeINT2TimeSeries (series, first, length))
		XLAL_ERROR_NULL(func, XLAL_EFUNC);

	return series;
}


INT2TimeSeries *XLALAddINT2TimeSeries (
	INT2TimeSeries *arg1,
	const INT2TimeSeries *arg2
)
{
	static const char func[] = "XLALAddINT2TimeSeries";
	REAL8 Delta_epoch = XLALGPSDiff(&arg2->epoch, &arg1->epoch);
	/* number of arg1 units per arg2 unit.  XLALUnitRatio() returns the
	 * number one obtains when one divides 1 of the first argument by 1
	 * of the second argument, for example if arg2 is in m and arg1 is
	 * in cm then unit_ratio = 100.0 */
	REAL8 unit_ratio = XLALUnitRatio(&arg2->sampleUnits, &arg1->sampleUnits);
	unsigned i, j;

	/* make sure arguments are compatible */
	if(XLALIsREAL8FailNaN(unit_ratio)) {
		XLALPrintError("%s(): incompatible units\n", func);
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	}
	if(arg1->f0 != arg2->f0) {
		XLALPrintError("%s(): incompatible heterodyne frequencies\n", func);
		XLAL_ERROR_NULL(func, XLAL_EDATA);
	}
	if(arg1->deltaT != arg2->deltaT) {
		XLALPrintError("%s(): incompatible sample periods\n", func);
		XLAL_ERROR_NULL(func, XLAL_EDATA);
	}

	/* set start indexes */
	if(Delta_epoch >= 0) {
		i = floor(Delta_epoch / arg1->deltaT + 0.5);
		j = 0;
	} else {
		i = 0;
		j = floor(-Delta_epoch / arg2->deltaT + 0.5);
	}

	/* add arg2 to arg1, adjusting the units */
	for(; i < arg1->data->length && j < arg2->data->length; i++, j++) {
		arg1->data->data[i] += arg2->data->data[j] * unit_ratio;
	}

	return arg1;
}







void XLALDestroyUINT2TimeSeries (
	UINT2TimeSeries *series
)
{
	if(series)
		XLALDestroyUINT2Sequence (series->data);
	XLALFree(series);
}


UINT2TimeSeries *XLALCreateUINT2TimeSeries (
	const CHAR *name,
	const LIGOTimeGPS *epoch,
	REAL8 f0,
	REAL8 deltaT,
	const LALUnit *sampleUnits,
	size_t length
)
{
	static const char func[] = "XLALCreateUINT2TimeSeries";
	UINT2TimeSeries *new;
	UINT2Sequence *sequence;

	new = XLALMalloc(sizeof(*new));
	sequence = XLALCreateUINT2Sequence (length);
	if(!new || !sequence) {
		XLALFree(new);
		XLALDestroyUINT2Sequence (sequence);
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	}

	if(name)
		strncpy(new->name, name, LALNameLength);
	else
		new->name[0] = '\0';
	new->epoch = *epoch;
	new->f0 = f0;
	new->deltaT = deltaT;
	new->sampleUnits = *sampleUnits;
	new->data = sequence;

	return new;
}


UINT2TimeSeries *XLALCutUINT2TimeSeries (
	const UINT2TimeSeries *series,
	size_t first,
	size_t length
)
{
	static const char func[] = "XLALCutUINT2TimeSeries";
	UINT2TimeSeries *new;
	UINT2Sequence *sequence;

	new = XLALMalloc(sizeof(*new));
	sequence = XLALCutUINT2Sequence (series->data, first, length);
	if(!new || !sequence) {
		XLALFree(new);
		XLALDestroyUINT2Sequence (sequence);
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	}

	*new = *series;
	new->data = sequence;
	XLALGPSAdd(&new->epoch, first * new->deltaT);

	return new;
}


UINT2TimeSeries *XLALResizeUINT2TimeSeries (
	UINT2TimeSeries *series,
	int first,
	size_t length
)
{
	static const char func[] = "XLALResizeUINT2TimeSeries";

	if(!XLALResizeUINT2Sequence (series->data, first, length))
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	XLALGPSAdd(&series->epoch, first * series->deltaT);

	return series;
}


UINT2TimeSeries *XLALShrinkUINT2TimeSeries (
	UINT2TimeSeries *series,
	size_t first,
	size_t length
)
{
	static const char func[] = "XLALShrinkUINT2TimeSeries";

	if(!XLALResizeUINT2TimeSeries (series, first, length))
		XLAL_ERROR_NULL(func, XLAL_EFUNC);

	return series;
}


UINT2TimeSeries *XLALAddUINT2TimeSeries (
	UINT2TimeSeries *arg1,
	const UINT2TimeSeries *arg2
)
{
	static const char func[] = "XLALAddUINT2TimeSeries";
	REAL8 Delta_epoch = XLALGPSDiff(&arg2->epoch, &arg1->epoch);
	/* number of arg1 units per arg2 unit.  XLALUnitRatio() returns the
	 * number one obtains when one divides 1 of the first argument by 1
	 * of the second argument, for example if arg2 is in m and arg1 is
	 * in cm then unit_ratio = 100.0 */
	REAL8 unit_ratio = XLALUnitRatio(&arg2->sampleUnits, &arg1->sampleUnits);
	unsigned i, j;

	/* make sure arguments are compatible */
	if(XLALIsREAL8FailNaN(unit_ratio)) {
		XLALPrintError("%s(): incompatible units\n", func);
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	}
	if(arg1->f0 != arg2->f0) {
		XLALPrintError("%s(): incompatible heterodyne frequencies\n", func);
		XLAL_ERROR_NULL(func, XLAL_EDATA);
	}
	if(arg1->deltaT != arg2->deltaT) {
		XLALPrintError("%s(): incompatible sample periods\n", func);
		XLAL_ERROR_NULL(func, XLAL_EDATA);
	}

	/* set start indexes */
	if(Delta_epoch >= 0) {
		i = floor(Delta_epoch / arg1->deltaT + 0.5);
		j = 0;
	} else {
		i = 0;
		j = floor(-Delta_epoch / arg2->deltaT + 0.5);
	}

	/* add arg2 to arg1, adjusting the units */
	for(; i < arg1->data->length && j < arg2->data->length; i++, j++) {
		arg1->data->data[i] += arg2->data->data[j] * unit_ratio;
	}

	return arg1;
}







void XLALDestroyINT4TimeSeries (
	INT4TimeSeries *series
)
{
	if(series)
		XLALDestroyINT4Sequence (series->data);
	XLALFree(series);
}


INT4TimeSeries *XLALCreateINT4TimeSeries (
	const CHAR *name,
	const LIGOTimeGPS *epoch,
	REAL8 f0,
	REAL8 deltaT,
	const LALUnit *sampleUnits,
	size_t length
)
{
	static const char func[] = "XLALCreateINT4TimeSeries";
	INT4TimeSeries *new;
	INT4Sequence *sequence;

	new = XLALMalloc(sizeof(*new));
	sequence = XLALCreateINT4Sequence (length);
	if(!new || !sequence) {
		XLALFree(new);
		XLALDestroyINT4Sequence (sequence);
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	}

	if(name)
		strncpy(new->name, name, LALNameLength);
	else
		new->name[0] = '\0';
	new->epoch = *epoch;
	new->f0 = f0;
	new->deltaT = deltaT;
	new->sampleUnits = *sampleUnits;
	new->data = sequence;

	return new;
}


INT4TimeSeries *XLALCutINT4TimeSeries (
	const INT4TimeSeries *series,
	size_t first,
	size_t length
)
{
	static const char func[] = "XLALCutINT4TimeSeries";
	INT4TimeSeries *new;
	INT4Sequence *sequence;

	new = XLALMalloc(sizeof(*new));
	sequence = XLALCutINT4Sequence (series->data, first, length);
	if(!new || !sequence) {
		XLALFree(new);
		XLALDestroyINT4Sequence (sequence);
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	}

	*new = *series;
	new->data = sequence;
	XLALGPSAdd(&new->epoch, first * new->deltaT);

	return new;
}


INT4TimeSeries *XLALResizeINT4TimeSeries (
	INT4TimeSeries *series,
	int first,
	size_t length
)
{
	static const char func[] = "XLALResizeINT4TimeSeries";

	if(!XLALResizeINT4Sequence (series->data, first, length))
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	XLALGPSAdd(&series->epoch, first * series->deltaT);

	return series;
}


INT4TimeSeries *XLALShrinkINT4TimeSeries (
	INT4TimeSeries *series,
	size_t first,
	size_t length
)
{
	static const char func[] = "XLALShrinkINT4TimeSeries";

	if(!XLALResizeINT4TimeSeries (series, first, length))
		XLAL_ERROR_NULL(func, XLAL_EFUNC);

	return series;
}


INT4TimeSeries *XLALAddINT4TimeSeries (
	INT4TimeSeries *arg1,
	const INT4TimeSeries *arg2
)
{
	static const char func[] = "XLALAddINT4TimeSeries";
	REAL8 Delta_epoch = XLALGPSDiff(&arg2->epoch, &arg1->epoch);
	/* number of arg1 units per arg2 unit.  XLALUnitRatio() returns the
	 * number one obtains when one divides 1 of the first argument by 1
	 * of the second argument, for example if arg2 is in m and arg1 is
	 * in cm then unit_ratio = 100.0 */
	REAL8 unit_ratio = XLALUnitRatio(&arg2->sampleUnits, &arg1->sampleUnits);
	unsigned i, j;

	/* make sure arguments are compatible */
	if(XLALIsREAL8FailNaN(unit_ratio)) {
		XLALPrintError("%s(): incompatible units\n", func);
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	}
	if(arg1->f0 != arg2->f0) {
		XLALPrintError("%s(): incompatible heterodyne frequencies\n", func);
		XLAL_ERROR_NULL(func, XLAL_EDATA);
	}
	if(arg1->deltaT != arg2->deltaT) {
		XLALPrintError("%s(): incompatible sample periods\n", func);
		XLAL_ERROR_NULL(func, XLAL_EDATA);
	}

	/* set start indexes */
	if(Delta_epoch >= 0) {
		i = floor(Delta_epoch / arg1->deltaT + 0.5);
		j = 0;
	} else {
		i = 0;
		j = floor(-Delta_epoch / arg2->deltaT + 0.5);
	}

	/* add arg2 to arg1, adjusting the units */
	for(; i < arg1->data->length && j < arg2->data->length; i++, j++) {
		arg1->data->data[i] += arg2->data->data[j] * unit_ratio;
	}

	return arg1;
}







void XLALDestroyUINT4TimeSeries (
	UINT4TimeSeries *series
)
{
	if(series)
		XLALDestroyUINT4Sequence (series->data);
	XLALFree(series);
}


UINT4TimeSeries *XLALCreateUINT4TimeSeries (
	const CHAR *name,
	const LIGOTimeGPS *epoch,
	REAL8 f0,
	REAL8 deltaT,
	const LALUnit *sampleUnits,
	size_t length
)
{
	static const char func[] = "XLALCreateUINT4TimeSeries";
	UINT4TimeSeries *new;
	UINT4Sequence *sequence;

	new = XLALMalloc(sizeof(*new));
	sequence = XLALCreateUINT4Sequence (length);
	if(!new || !sequence) {
		XLALFree(new);
		XLALDestroyUINT4Sequence (sequence);
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	}

	if(name)
		strncpy(new->name, name, LALNameLength);
	else
		new->name[0] = '\0';
	new->epoch = *epoch;
	new->f0 = f0;
	new->deltaT = deltaT;
	new->sampleUnits = *sampleUnits;
	new->data = sequence;

	return new;
}


UINT4TimeSeries *XLALCutUINT4TimeSeries (
	const UINT4TimeSeries *series,
	size_t first,
	size_t length
)
{
	static const char func[] = "XLALCutUINT4TimeSeries";
	UINT4TimeSeries *new;
	UINT4Sequence *sequence;

	new = XLALMalloc(sizeof(*new));
	sequence = XLALCutUINT4Sequence (series->data, first, length);
	if(!new || !sequence) {
		XLALFree(new);
		XLALDestroyUINT4Sequence (sequence);
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	}

	*new = *series;
	new->data = sequence;
	XLALGPSAdd(&new->epoch, first * new->deltaT);

	return new;
}


UINT4TimeSeries *XLALResizeUINT4TimeSeries (
	UINT4TimeSeries *series,
	int first,
	size_t length
)
{
	static const char func[] = "XLALResizeUINT4TimeSeries";

	if(!XLALResizeUINT4Sequence (series->data, first, length))
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	XLALGPSAdd(&series->epoch, first * series->deltaT);

	return series;
}


UINT4TimeSeries *XLALShrinkUINT4TimeSeries (
	UINT4TimeSeries *series,
	size_t first,
	size_t length
)
{
	static const char func[] = "XLALShrinkUINT4TimeSeries";

	if(!XLALResizeUINT4TimeSeries (series, first, length))
		XLAL_ERROR_NULL(func, XLAL_EFUNC);

	return series;
}


UINT4TimeSeries *XLALAddUINT4TimeSeries (
	UINT4TimeSeries *arg1,
	const UINT4TimeSeries *arg2
)
{
	static const char func[] = "XLALAddUINT4TimeSeries";
	REAL8 Delta_epoch = XLALGPSDiff(&arg2->epoch, &arg1->epoch);
	/* number of arg1 units per arg2 unit.  XLALUnitRatio() returns the
	 * number one obtains when one divides 1 of the first argument by 1
	 * of the second argument, for example if arg2 is in m and arg1 is
	 * in cm then unit_ratio = 100.0 */
	REAL8 unit_ratio = XLALUnitRatio(&arg2->sampleUnits, &arg1->sampleUnits);
	unsigned i, j;

	/* make sure arguments are compatible */
	if(XLALIsREAL8FailNaN(unit_ratio)) {
		XLALPrintError("%s(): incompatible units\n", func);
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	}
	if(arg1->f0 != arg2->f0) {
		XLALPrintError("%s(): incompatible heterodyne frequencies\n", func);
		XLAL_ERROR_NULL(func, XLAL_EDATA);
	}
	if(arg1->deltaT != arg2->deltaT) {
		XLALPrintError("%s(): incompatible sample periods\n", func);
		XLAL_ERROR_NULL(func, XLAL_EDATA);
	}

	/* set start indexes */
	if(Delta_epoch >= 0) {
		i = floor(Delta_epoch / arg1->deltaT + 0.5);
		j = 0;
	} else {
		i = 0;
		j = floor(-Delta_epoch / arg2->deltaT + 0.5);
	}

	/* add arg2 to arg1, adjusting the units */
	for(; i < arg1->data->length && j < arg2->data->length; i++, j++) {
		arg1->data->data[i] += arg2->data->data[j] * unit_ratio;
	}

	return arg1;
}







void XLALDestroyINT8TimeSeries (
	INT8TimeSeries *series
)
{
	if(series)
		XLALDestroyINT8Sequence (series->data);
	XLALFree(series);
}


INT8TimeSeries *XLALCreateINT8TimeSeries (
	const CHAR *name,
	const LIGOTimeGPS *epoch,
	REAL8 f0,
	REAL8 deltaT,
	const LALUnit *sampleUnits,
	size_t length
)
{
	static const char func[] = "XLALCreateINT8TimeSeries";
	INT8TimeSeries *new;
	INT8Sequence *sequence;

	new = XLALMalloc(sizeof(*new));
	sequence = XLALCreateINT8Sequence (length);
	if(!new || !sequence) {
		XLALFree(new);
		XLALDestroyINT8Sequence (sequence);
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	}

	if(name)
		strncpy(new->name, name, LALNameLength);
	else
		new->name[0] = '\0';
	new->epoch = *epoch;
	new->f0 = f0;
	new->deltaT = deltaT;
	new->sampleUnits = *sampleUnits;
	new->data = sequence;

	return new;
}


INT8TimeSeries *XLALCutINT8TimeSeries (
	const INT8TimeSeries *series,
	size_t first,
	size_t length
)
{
	static const char func[] = "XLALCutINT8TimeSeries";
	INT8TimeSeries *new;
	INT8Sequence *sequence;

	new = XLALMalloc(sizeof(*new));
	sequence = XLALCutINT8Sequence (series->data, first, length);
	if(!new || !sequence) {
		XLALFree(new);
		XLALDestroyINT8Sequence (sequence);
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	}

	*new = *series;
	new->data = sequence;
	XLALGPSAdd(&new->epoch, first * new->deltaT);

	return new;
}


INT8TimeSeries *XLALResizeINT8TimeSeries (
	INT8TimeSeries *series,
	int first,
	size_t length
)
{
	static const char func[] = "XLALResizeINT8TimeSeries";

	if(!XLALResizeINT8Sequence (series->data, first, length))
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	XLALGPSAdd(&series->epoch, first * series->deltaT);

	return series;
}


INT8TimeSeries *XLALShrinkINT8TimeSeries (
	INT8TimeSeries *series,
	size_t first,
	size_t length
)
{
	static const char func[] = "XLALShrinkINT8TimeSeries";

	if(!XLALResizeINT8TimeSeries (series, first, length))
		XLAL_ERROR_NULL(func, XLAL_EFUNC);

	return series;
}


INT8TimeSeries *XLALAddINT8TimeSeries (
	INT8TimeSeries *arg1,
	const INT8TimeSeries *arg2
)
{
	static const char func[] = "XLALAddINT8TimeSeries";
	REAL8 Delta_epoch = XLALGPSDiff(&arg2->epoch, &arg1->epoch);
	/* number of arg1 units per arg2 unit.  XLALUnitRatio() returns the
	 * number one obtains when one divides 1 of the first argument by 1
	 * of the second argument, for example if arg2 is in m and arg1 is
	 * in cm then unit_ratio = 100.0 */
	REAL8 unit_ratio = XLALUnitRatio(&arg2->sampleUnits, &arg1->sampleUnits);
	unsigned i, j;

	/* make sure arguments are compatible */
	if(XLALIsREAL8FailNaN(unit_ratio)) {
		XLALPrintError("%s(): incompatible units\n", func);
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	}
	if(arg1->f0 != arg2->f0) {
		XLALPrintError("%s(): incompatible heterodyne frequencies\n", func);
		XLAL_ERROR_NULL(func, XLAL_EDATA);
	}
	if(arg1->deltaT != arg2->deltaT) {
		XLALPrintError("%s(): incompatible sample periods\n", func);
		XLAL_ERROR_NULL(func, XLAL_EDATA);
	}

	/* set start indexes */
	if(Delta_epoch >= 0) {
		i = floor(Delta_epoch / arg1->deltaT + 0.5);
		j = 0;
	} else {
		i = 0;
		j = floor(-Delta_epoch / arg2->deltaT + 0.5);
	}

	/* add arg2 to arg1, adjusting the units */
	for(; i < arg1->data->length && j < arg2->data->length; i++, j++) {
		arg1->data->data[i] += arg2->data->data[j] * unit_ratio;
	}

	return arg1;
}







void XLALDestroyUINT8TimeSeries (
	UINT8TimeSeries *series
)
{
	if(series)
		XLALDestroyUINT8Sequence (series->data);
	XLALFree(series);
}


UINT8TimeSeries *XLALCreateUINT8TimeSeries (
	const CHAR *name,
	const LIGOTimeGPS *epoch,
	REAL8 f0,
	REAL8 deltaT,
	const LALUnit *sampleUnits,
	size_t length
)
{
	static const char func[] = "XLALCreateUINT8TimeSeries";
	UINT8TimeSeries *new;
	UINT8Sequence *sequence;

	new = XLALMalloc(sizeof(*new));
	sequence = XLALCreateUINT8Sequence (length);
	if(!new || !sequence) {
		XLALFree(new);
		XLALDestroyUINT8Sequence (sequence);
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	}

	if(name)
		strncpy(new->name, name, LALNameLength);
	else
		new->name[0] = '\0';
	new->epoch = *epoch;
	new->f0 = f0;
	new->deltaT = deltaT;
	new->sampleUnits = *sampleUnits;
	new->data = sequence;

	return new;
}


UINT8TimeSeries *XLALCutUINT8TimeSeries (
	const UINT8TimeSeries *series,
	size_t first,
	size_t length
)
{
	static const char func[] = "XLALCutUINT8TimeSeries";
	UINT8TimeSeries *new;
	UINT8Sequence *sequence;

	new = XLALMalloc(sizeof(*new));
	sequence = XLALCutUINT8Sequence (series->data, first, length);
	if(!new || !sequence) {
		XLALFree(new);
		XLALDestroyUINT8Sequence (sequence);
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	}

	*new = *series;
	new->data = sequence;
	XLALGPSAdd(&new->epoch, first * new->deltaT);

	return new;
}


UINT8TimeSeries *XLALResizeUINT8TimeSeries (
	UINT8TimeSeries *series,
	int first,
	size_t length
)
{
	static const char func[] = "XLALResizeUINT8TimeSeries";

	if(!XLALResizeUINT8Sequence (series->data, first, length))
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	XLALGPSAdd(&series->epoch, first * series->deltaT);

	return series;
}


UINT8TimeSeries *XLALShrinkUINT8TimeSeries (
	UINT8TimeSeries *series,
	size_t first,
	size_t length
)
{
	static const char func[] = "XLALShrinkUINT8TimeSeries";

	if(!XLALResizeUINT8TimeSeries (series, first, length))
		XLAL_ERROR_NULL(func, XLAL_EFUNC);

	return series;
}


UINT8TimeSeries *XLALAddUINT8TimeSeries (
	UINT8TimeSeries *arg1,
	const UINT8TimeSeries *arg2
)
{
	static const char func[] = "XLALAddUINT8TimeSeries";
	REAL8 Delta_epoch = XLALGPSDiff(&arg2->epoch, &arg1->epoch);
	/* number of arg1 units per arg2 unit.  XLALUnitRatio() returns the
	 * number one obtains when one divides 1 of the first argument by 1
	 * of the second argument, for example if arg2 is in m and arg1 is
	 * in cm then unit_ratio = 100.0 */
	REAL8 unit_ratio = XLALUnitRatio(&arg2->sampleUnits, &arg1->sampleUnits);
	unsigned i, j;

	/* make sure arguments are compatible */
	if(XLALIsREAL8FailNaN(unit_ratio)) {
		XLALPrintError("%s(): incompatible units\n", func);
		XLAL_ERROR_NULL(func, XLAL_EFUNC);
	}
	if(arg1->f0 != arg2->f0) {
		XLALPrintError("%s(): incompatible heterodyne frequencies\n", func);
		XLAL_ERROR_NULL(func, XLAL_EDATA);
	}
	if(arg1->deltaT != arg2->deltaT) {
		XLALPrintError("%s(): incompatible sample periods\n", func);
		XLAL_ERROR_NULL(func, XLAL_EDATA);
	}

	/* set start indexes */
	if(Delta_epoch >= 0) {
		i = floor(Delta_epoch / arg1->deltaT + 0.5);
		j = 0;
	} else {
		i = 0;
		j = floor(-Delta_epoch / arg2->deltaT + 0.5);
	}

	/* add arg2 to arg1, adjusting the units */
	for(; i < arg1->data->length && j < arg2->data->length; i++, j++) {
		arg1->data->data[i] += arg2->data->data[j] * unit_ratio;
	}

	return arg1;
}


